from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions
from .....Internal.Utilities import trim_str_response
from ..... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class DexchangeCls:
	"""Dexchange commands group definition. 8 total commands, 2 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("dexchange", core, parent)

	@property
	def afile(self):
		"""afile commands group. 1 Sub-classes, 3 commands."""
		if not hasattr(self, '_afile'):
			from .Afile import AfileCls
			self._afile = AfileCls(self._core, self._cmd_group)
		return self._afile

	@property
	def execute(self):
		"""execute commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_execute'):
			from .Execute import ExecuteCls
			self._execute = ExecuteCls(self._core, self._cmd_group)
		return self._execute

	# noinspection PyTypeChecker
	def get_mode(self) -> enums.DexchMode:
		"""SCPI: [SOURce<HW>]:CORRection:DEXChange:MODE \n
		Snippet: value: enums.DexchMode = driver.source.correction.dexchange.get_mode() \n
		Selects if user correction lists should be imported or exported. Depending on the selection her, the file select command
		define either the source or the destination for user correction lists and ASCII files. \n
			:return: mode: IMPort| EXPort
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:CORRection:DEXChange:MODE?')
		return Conversions.str_to_scalar_enum(response, enums.DexchMode)

	def set_mode(self, mode: enums.DexchMode) -> None:
		"""SCPI: [SOURce<HW>]:CORRection:DEXChange:MODE \n
		Snippet: driver.source.correction.dexchange.set_mode(mode = enums.DexchMode.EXPort) \n
		Selects if user correction lists should be imported or exported. Depending on the selection her, the file select command
		define either the source or the destination for user correction lists and ASCII files. \n
			:param mode: IMPort| EXPort
		"""
		param = Conversions.enum_scalar_to_str(mode, enums.DexchMode)
		self._core.io.write(f'SOURce<HwInstance>:CORRection:DEXChange:MODE {param}')

	def get_select(self) -> str:
		"""SCPI: [SOURce<HW>]:CORRection:DEXChange:SELect \n
		Snippet: value: str = driver.source.correction.dexchange.get_select() \n
		Selects the user correction list to be imported or exported. The user correction files are stored with the fixed file
		extensions *.uco in a directory of the user's choice. The directory applicable to the commands is defined with the
		command method RsSgt.MassMemory.currentDirectory. A path can also be specified in command SOUR:CORR:DEXC:SEL, in which
		case the files are stored or loaded in the specified directory. \n
			:return: filename: string
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:CORRection:DEXChange:SELect?')
		return trim_str_response(response)

	def set_select(self, filename: str) -> None:
		"""SCPI: [SOURce<HW>]:CORRection:DEXChange:SELect \n
		Snippet: driver.source.correction.dexchange.set_select(filename = '1') \n
		Selects the user correction list to be imported or exported. The user correction files are stored with the fixed file
		extensions *.uco in a directory of the user's choice. The directory applicable to the commands is defined with the
		command method RsSgt.MassMemory.currentDirectory. A path can also be specified in command SOUR:CORR:DEXC:SEL, in which
		case the files are stored or loaded in the specified directory. \n
			:param filename: string
		"""
		param = Conversions.value_to_quoted_str(filename)
		self._core.io.write(f'SOURce<HwInstance>:CORRection:DEXChange:SELect {param}')

	def clone(self) -> 'DexchangeCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = DexchangeCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup
from ...Internal import Conversions
from ...Internal.Utilities import trim_str_response


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class LevelCls:
	"""Level commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("level", core, parent)

	def get_temperature(self) -> str:
		"""SCPI: CALibration:LEVel:TEMPerature \n
		Snippet: value: str = driver.calibration.level.get_temperature() \n
		Queries the delta temperature since the last performed adjustment. \n
			:return: temperature: string
		"""
		response = self._core.io.query_str('CALibration:LEVel:TEMPerature?')
		return trim_str_response(response)

	def get_measure(self) -> bool:
		"""SCPI: CALibration:LEVel:[MEASure] \n
		Snippet: value: bool = driver.calibration.level.get_measure() \n
		Starts all adjustments which affect the level. \n
			:return: level: 1| ON| 0| OFF
		"""
		response = self._core.io.query_str('CALibration:LEVel:MEASure?')
		return Conversions.str_to_bool(response)



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class FrequencyCls:
	"""Frequency commands group definition. 9 total commands, 2 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("frequency", core, parent)

	@property
	def cw(self):
		"""cw commands group. 1 Sub-classes, 2 commands."""
		if not hasattr(self, '_cw'):
			from .Cw import CwCls
			self._cw = CwCls(self._core, self._cmd_group)
		return self._cw

	@property
	def fixed(self):
		"""fixed commands group. 1 Sub-classes, 2 commands."""
		if not hasattr(self, '_fixed'):
			from .Fixed import FixedCls
			self._fixed = FixedCls(self._core, self._cmd_group)
		return self._fixed

	def get_offset(self) -> float:
		"""SCPI: [SOURce]:FREQuency:OFFSet \n
		Snippet: value: float = driver.source.frequency.get_offset() \n
		Sets a frequency offset, for example include the frequency shift of downstream instrument. Note: Enabled frequency offset
		affects the result of the query SOURce:FREQuency:CW? The query returns the frequency, including frequency offset. \n
			:return: offset: float Range: -3e9 to 3e9
		"""
		response = self._core.io.query_str('SOURce:FREQuency:OFFSet?')
		return Conversions.str_to_float(response)

	def set_offset(self, offset: float) -> None:
		"""SCPI: [SOURce]:FREQuency:OFFSet \n
		Snippet: driver.source.frequency.set_offset(offset = 1.0) \n
		Sets a frequency offset, for example include the frequency shift of downstream instrument. Note: Enabled frequency offset
		affects the result of the query SOURce:FREQuency:CW? The query returns the frequency, including frequency offset. \n
			:param offset: float Range: -3e9 to 3e9
		"""
		param = Conversions.decimal_value_to_str(offset)
		self._core.io.write(f'SOURce:FREQuency:OFFSet {param}')

	def clone(self) -> 'FrequencyCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = FrequencyCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class IqRatioCls:
	"""IqRatio commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("iqRatio", core, parent)

	def get_magnitude(self) -> float:
		"""SCPI: [SOURce<HW>]:IQ:IMPairment:IQRatio:[MAGNitude] \n
		Snippet: value: float = driver.source.iq.impairment.iqRatio.get_magnitude() \n
		Sets the ratio of I modulation to Q modulation (amplification “imbalance”) . The input may be either in dB or %.
		The resolution is 0.001 dB, an input in percent is rounded to the closest valid value in dB. A query returns the value in
		dB. \n
			:return: ipartq_ratio: No help available
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:IMPairment:IQRatio:MAGNitude?')
		return Conversions.str_to_float(response)

	def set_magnitude(self, ipartq_ratio: float) -> None:
		"""SCPI: [SOURce<HW>]:IQ:IMPairment:IQRatio:[MAGNitude] \n
		Snippet: driver.source.iq.impairment.iqRatio.set_magnitude(ipartq_ratio = 1.0) \n
		Sets the ratio of I modulation to Q modulation (amplification “imbalance”) . The input may be either in dB or %.
		The resolution is 0.001 dB, an input in percent is rounded to the closest valid value in dB. A query returns the value in
		dB. \n
			:param ipartq_ratio: float Range: -1 to 1
		"""
		param = Conversions.decimal_value_to_str(ipartq_ratio)
		self._core.io.write(f'SOURce<HwInstance>:IQ:IMPairment:IQRatio:MAGNitude {param}')



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions
from ..... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class TriggerCls:
	"""Trigger commands group definition. 4 total commands, 1 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("trigger", core, parent)

	@property
	def external(self):
		"""external commands group. 1 Sub-classes, 2 commands."""
		if not hasattr(self, '_external'):
			from .External import ExternalCls
			self._external = ExternalCls(self._core, self._cmd_group)
		return self._external

	# noinspection PyTypeChecker
	def get_mode(self) -> enums.PulsTrigMode:
		"""SCPI: [SOURce<HW>]:PULM:TRIGger:MODE \n
		Snippet: value: enums.PulsTrigMode = driver.source.pulm.trigger.get_mode() \n
		Selects the trigger mode for pulse modulation. \n
			:return: mode: AUTO| EXTernal| EGATe AUTO The pulse modulation is generated continuously. EXTernal The pulse modulation is triggered by an external trigger event. The trigger signal is supplied via the trigger connector. EGATe The pulse modulation is gated by an external gate signal. The trigger signal is supplied via the trigger connector.
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:PULM:TRIGger:MODE?')
		return Conversions.str_to_scalar_enum(response, enums.PulsTrigMode)

	def set_mode(self, mode: enums.PulsTrigMode) -> None:
		"""SCPI: [SOURce<HW>]:PULM:TRIGger:MODE \n
		Snippet: driver.source.pulm.trigger.set_mode(mode = enums.PulsTrigMode.AUTO) \n
		Selects the trigger mode for pulse modulation. \n
			:param mode: AUTO| EXTernal| EGATe AUTO The pulse modulation is generated continuously. EXTernal The pulse modulation is triggered by an external trigger event. The trigger signal is supplied via the trigger connector. EGATe The pulse modulation is gated by an external gate signal. The trigger signal is supplied via the trigger connector.
		"""
		param = Conversions.enum_scalar_to_str(mode, enums.PulsTrigMode)
		self._core.io.write(f'SOURce<HwInstance>:PULM:TRIGger:MODE {param}')

	def clone(self) -> 'TriggerCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = TriggerCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup
from ......Internal import Conversions
from ......Internal.Utilities import trim_str_response
from ...... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class IpAddressCls:
	"""IpAddress commands group definition. 5 total commands, 1 Subgroups, 4 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("ipAddress", core, parent)

	@property
	def subnet(self):
		"""subnet commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_subnet'):
			from .Subnet import SubnetCls
			self._subnet = SubnetCls(self._core, self._cmd_group)
		return self._subnet

	# noinspection PyTypeChecker
	def get_mode(self) -> enums.NetMode:
		"""SCPI: SYSTem:COMMunicate:NETWork:IPADdress:MODE \n
		Snippet: value: enums.NetMode = driver.system.communicate.network.ipAddress.get_mode() \n
		Selects manual or automatic setting of the IP address. \n
			:return: mode: AUTO| STATic
		"""
		response = self._core.io.query_str('SYSTem:COMMunicate:NETWork:IPADdress:MODE?')
		return Conversions.str_to_scalar_enum(response, enums.NetMode)

	def set_mode(self, mode: enums.NetMode) -> None:
		"""SCPI: SYSTem:COMMunicate:NETWork:IPADdress:MODE \n
		Snippet: driver.system.communicate.network.ipAddress.set_mode(mode = enums.NetMode.AUTO) \n
		Selects manual or automatic setting of the IP address. \n
			:param mode: AUTO| STATic
		"""
		param = Conversions.enum_scalar_to_str(mode, enums.NetMode)
		self._core.io.write(f'SYSTem:COMMunicate:NETWork:IPADdress:MODE {param}')

	def get_dns(self) -> str:
		"""SCPI: SYSTem:COMMunicate:NETWork:[IPADdress]:DNS \n
		Snippet: value: str = driver.system.communicate.network.ipAddress.get_dns() \n
		Determines or queries the network DNS server to resolve the name. \n
			:return: dns: string
		"""
		response = self._core.io.query_str('SYSTem:COMMunicate:NETWork:IPADdress:DNS?')
		return trim_str_response(response)

	def set_dns(self, dns: str) -> None:
		"""SCPI: SYSTem:COMMunicate:NETWork:[IPADdress]:DNS \n
		Snippet: driver.system.communicate.network.ipAddress.set_dns(dns = '1') \n
		Determines or queries the network DNS server to resolve the name. \n
			:param dns: string
		"""
		param = Conversions.value_to_quoted_str(dns)
		self._core.io.write(f'SYSTem:COMMunicate:NETWork:IPADdress:DNS {param}')

	def get_gateway(self) -> str:
		"""SCPI: SYSTem:COMMunicate:NETWork:[IPADdress]:GATeway \n
		Snippet: value: str = driver.system.communicate.network.ipAddress.get_gateway() \n
		Sets the IP address of the default gateway. \n
			:return: gateway: string Range: 0.0.0.0 to ff.ff.ff.ff
		"""
		response = self._core.io.query_str('SYSTem:COMMunicate:NETWork:IPADdress:GATeway?')
		return trim_str_response(response)

	def set_gateway(self, gateway: str) -> None:
		"""SCPI: SYSTem:COMMunicate:NETWork:[IPADdress]:GATeway \n
		Snippet: driver.system.communicate.network.ipAddress.set_gateway(gateway = '1') \n
		Sets the IP address of the default gateway. \n
			:param gateway: string Range: 0.0.0.0 to ff.ff.ff.ff
		"""
		param = Conversions.value_to_quoted_str(gateway)
		self._core.io.write(f'SYSTem:COMMunicate:NETWork:IPADdress:GATeway {param}')

	def get_value(self) -> str:
		"""SCPI: SYSTem:COMMunicate:NETWork:IPADdress \n
		Snippet: value: str = driver.system.communicate.network.ipAddress.get_value() \n
		Sets the IP address. \n
			:return: ip_address: string Range: 0.0.0.0. to ff.ff.ff.ff
		"""
		response = self._core.io.query_str('SYSTem:COMMunicate:NETWork:IPADdress?')
		return trim_str_response(response)

	def set_value(self, ip_address: str) -> None:
		"""SCPI: SYSTem:COMMunicate:NETWork:IPADdress \n
		Snippet: driver.system.communicate.network.ipAddress.set_value(ip_address = '1') \n
		Sets the IP address. \n
			:param ip_address: string Range: 0.0.0.0. to ff.ff.ff.ff
		"""
		param = Conversions.value_to_quoted_str(ip_address)
		self._core.io.write(f'SYSTem:COMMunicate:NETWork:IPADdress {param}')

	def clone(self) -> 'IpAddressCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = IpAddressCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions
from ..... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class FixedCls:
	"""Fixed commands group definition. 4 total commands, 1 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("fixed", core, parent)

	@property
	def step(self):
		"""step commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_step'):
			from .Step import StepCls
			self._step = StepCls(self._core, self._cmd_group)
		return self._step

	# noinspection PyTypeChecker
	def get_recall(self) -> enums.InclExcl:
		"""SCPI: [SOURce<HW>]:FREQuency:[FIXed]:RCL \n
		Snippet: value: enums.InclExcl = driver.source.frequency.fixed.get_recall() \n
		No command help available \n
			:return: rcl_excl_freq: No help available
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:FREQuency:FIXed:RCL?')
		return Conversions.str_to_scalar_enum(response, enums.InclExcl)

	def set_recall(self, rcl_excl_freq: enums.InclExcl) -> None:
		"""SCPI: [SOURce<HW>]:FREQuency:[FIXed]:RCL \n
		Snippet: driver.source.frequency.fixed.set_recall(rcl_excl_freq = enums.InclExcl.EXCLude) \n
		No command help available \n
			:param rcl_excl_freq: No help available
		"""
		param = Conversions.enum_scalar_to_str(rcl_excl_freq, enums.InclExcl)
		self._core.io.write(f'SOURce<HwInstance>:FREQuency:FIXed:RCL {param}')

	def get_value(self) -> float:
		"""SCPI: [SOURce<HW>]:FREQuency:[FIXed] \n
		Snippet: value: float = driver.source.frequency.fixed.get_value() \n
		Sets the RF frequency at the RF output connector of the selected instrument. Note: Enabled frequency offset affects the
		result of this query. The query returns the frequency, including frequency offset. \n
			:return: cw: float
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:FREQuency:FIXed?')
		return Conversions.str_to_float(response)

	def set_value(self, cw: float) -> None:
		"""SCPI: [SOURce<HW>]:FREQuency:[FIXed] \n
		Snippet: driver.source.frequency.fixed.set_value(cw = 1.0) \n
		Sets the RF frequency at the RF output connector of the selected instrument. Note: Enabled frequency offset affects the
		result of this query. The query returns the frequency, including frequency offset. \n
			:param cw: float
		"""
		param = Conversions.decimal_value_to_str(cw)
		self._core.io.write(f'SOURce<HwInstance>:FREQuency:FIXed {param}')

	def clone(self) -> 'FixedCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = FixedCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class OloadCls:
	"""Oload commands group definition. 3 total commands, 1 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("oload", core, parent)

	@property
	def hold(self):
		"""hold commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_hold'):
			from .Hold import HoldCls
			self._hold = HoldCls(self._core, self._cmd_group)
		return self._hold

	def get_state(self) -> bool:
		"""SCPI: [SOURce<HW>]:BBIN:OLOad:STATe \n
		Snippet: value: bool = driver.source.bbin.oload.get_state() \n
		This command queries the current overload state of the A/D converter. \n
			:return: state: 0| 1| OFF| ON
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BBIN:OLOad:STATe?')
		return Conversions.str_to_bool(response)

	def clone(self) -> 'OloadCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = OloadCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from ........Internal.Core import Core
from ........Internal.CommandsGroup import CommandsGroup
from ........ import repcap


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class SonceCls:
	"""Sonce commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("sonce", core, parent)

	def set(self, channel=repcap.Channel.Default) -> None:
		"""SCPI: [SOURce<HW>]:CORRection:CSET:DATA:[SENSor<CH>]:[POWer]:SONCe \n
		Snippet: driver.source.correction.cset.data.sensor.power.sonce.set(channel = repcap.Channel.Default) \n
		The command fills the selected user correction list with the level values measured by the power sensor for the given
		frequencies. To select the used power sensor set the suffix in key word SENSe. \n
			:param channel: optional repeated capability selector. Default value: Nr1 (settable in the interface 'Sensor')
		"""
		channel_cmd_val = self._cmd_group.get_repcap_cmd_value(channel, repcap.Channel)
		self._core.io.write(f'SOURce<HwInstance>:CORRection:CSET:DATA:SENSor{channel_cmd_val}:POWer:SONCe')

	def set_with_opc(self, channel=repcap.Channel.Default, opc_timeout_ms: int = -1) -> None:
		channel_cmd_val = self._cmd_group.get_repcap_cmd_value(channel, repcap.Channel)
		"""SCPI: [SOURce<HW>]:CORRection:CSET:DATA:[SENSor<CH>]:[POWer]:SONCe \n
		Snippet: driver.source.correction.cset.data.sensor.power.sonce.set_with_opc(channel = repcap.Channel.Default) \n
		The command fills the selected user correction list with the level values measured by the power sensor for the given
		frequencies. To select the used power sensor set the suffix in key word SENSe. \n
		Same as set, but waits for the operation to complete before continuing further. Use the RsSgt.utilities.opc_timeout_set() to set the timeout value. \n
			:param channel: optional repeated capability selector. Default value: Nr1 (settable in the interface 'Sensor')
			:param opc_timeout_ms: Maximum time to wait in milliseconds, valid only for this call."""
		self._core.io.write_with_opc(f'SOURce<HwInstance>:CORRection:CSET:DATA:SENSor{channel_cmd_val}:POWer:SONCe', opc_timeout_ms)



from ........Internal.Core import Core
from ........Internal.CommandsGroup import CommandsGroup
from ........Internal import Conversions
from ........ import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class DetroughingCls:
	"""Detroughing commands group definition. 4 total commands, 0 Subgroups, 4 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("detroughing", core, parent)

	def get_coupling(self) -> bool:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:SHAPing:DETRoughing:COUPling \n
		Snippet: value: bool = driver.source.iq.output.analog.envelope.shaping.detroughing.get_coupling() \n
		Enables/disables deriving the detroughing factor (d) from the selected Vcc value. \n
			:return: coupling_state: 1| ON| 0| OFF
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:SHAPing:DETRoughing:COUPling?')
		return Conversions.str_to_bool(response)

	def set_coupling(self, coupling_state: bool) -> None:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:SHAPing:DETRoughing:COUPling \n
		Snippet: driver.source.iq.output.analog.envelope.shaping.detroughing.set_coupling(coupling_state = False) \n
		Enables/disables deriving the detroughing factor (d) from the selected Vcc value. \n
			:param coupling_state: 1| ON| 0| OFF
		"""
		param = Conversions.bool_to_str(coupling_state)
		self._core.io.write(f'SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:SHAPing:DETRoughing:COUPling {param}')

	def get_factor(self) -> float:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:SHAPing:DETRoughing:FACTor \n
		Snippet: value: float = driver.source.iq.output.analog.envelope.shaping.detroughing.get_factor() \n
		Sets the detroughing factor. \n
			:return: detr_factor: float Range: 0 to 2
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:SHAPing:DETRoughing:FACTor?')
		return Conversions.str_to_float(response)

	def set_factor(self, detr_factor: float) -> None:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:SHAPing:DETRoughing:FACTor \n
		Snippet: driver.source.iq.output.analog.envelope.shaping.detroughing.set_factor(detr_factor = 1.0) \n
		Sets the detroughing factor. \n
			:param detr_factor: float Range: 0 to 2
		"""
		param = Conversions.decimal_value_to_str(detr_factor)
		self._core.io.write(f'SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:SHAPing:DETRoughing:FACTor {param}')

	# noinspection PyTypeChecker
	def get_function(self) -> enums.IqOutEnvDetrFunc:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:SHAPing:DETRoughing:FUNCtion \n
		Snippet: value: enums.IqOutEnvDetrFunc = driver.source.iq.output.analog.envelope.shaping.detroughing.get_function() \n
		Sets the detroughing function. \n
			:return: detr_function: F1| F2| F3
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:SHAPing:DETRoughing:FUNCtion?')
		return Conversions.str_to_scalar_enum(response, enums.IqOutEnvDetrFunc)

	def set_function(self, detr_function: enums.IqOutEnvDetrFunc) -> None:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:SHAPing:DETRoughing:FUNCtion \n
		Snippet: driver.source.iq.output.analog.envelope.shaping.detroughing.set_function(detr_function = enums.IqOutEnvDetrFunc.F1) \n
		Sets the detroughing function. \n
			:param detr_function: F1| F2| F3
		"""
		param = Conversions.enum_scalar_to_str(detr_function, enums.IqOutEnvDetrFunc)
		self._core.io.write(f'SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:SHAPing:DETRoughing:FUNCtion {param}')

	def get_pexponent(self) -> float:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:SHAPing:DETRoughing:PEXPonent \n
		Snippet: value: float = driver.source.iq.output.analog.envelope.shaping.detroughing.get_pexponent() \n
		Sets the exponent (a) for the detroughing function F3. \n
			:return: power_exponent: float Range: 1 to 10
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:SHAPing:DETRoughing:PEXPonent?')
		return Conversions.str_to_float(response)

	def set_pexponent(self, power_exponent: float) -> None:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:SHAPing:DETRoughing:PEXPonent \n
		Snippet: driver.source.iq.output.analog.envelope.shaping.detroughing.set_pexponent(power_exponent = 1.0) \n
		Sets the exponent (a) for the detroughing function F3. \n
			:param power_exponent: float Range: 1 to 10
		"""
		param = Conversions.decimal_value_to_str(power_exponent)
		self._core.io.write(f'SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:SHAPing:DETRoughing:PEXPonent {param}')



from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup
from ...Internal.Utilities import trim_str_response


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class DeviceCls:
	"""Device commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("device", core, parent)

	def get_id(self) -> str:
		"""SCPI: SYSTem:DEVice:ID \n
		Snippet: value: str = driver.system.device.get_id() \n
		No command help available \n
			:return: pseudo_string: No help available
		"""
		response = self._core.io.query_str('SYSTem:DEVice:ID?')
		return trim_str_response(response)



from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup
from ...Internal import Conversions
from ... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class OutputCls:
	"""Output commands group definition. 6 total commands, 3 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("output", core, parent)

	@property
	def afixed(self):
		"""afixed commands group. 1 Sub-classes, 0 commands."""
		if not hasattr(self, '_afixed'):
			from .Afixed import AfixedCls
			self._afixed = AfixedCls(self._core, self._cmd_group)
		return self._afixed

	@property
	def protection(self):
		"""protection commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_protection'):
			from .Protection import ProtectionCls
			self._protection = ProtectionCls(self._core, self._cmd_group)
		return self._protection

	@property
	def state(self):
		"""state commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_state'):
			from .State import StateCls
			self._state = StateCls(self._core, self._cmd_group)
		return self._state

	# noinspection PyTypeChecker
	def get_amode(self) -> enums.AttMode:
		"""SCPI: OUTPut<HW>:AMODe \n
		Snippet: value: enums.AttMode = driver.output.get_amode() \n
		Switches the mode of the attenuator at the RF output. \n
			:return: amode: AUTO| FIXed| APASsive AUTO The attenuator is switched automatically. The level settings are made in the full range. APASsive The attenuator is switched automatically. The level settings are made only for the passive reference circuits. The high-level ranges are not available. FIXed The level settings are made without switching the attenuator. When this operating mode is switched on, the attenuator is fixed to its current position and the resulting variation range is defined.
		"""
		response = self._core.io.query_str('OUTPut<HwInstance>:AMODe?')
		return Conversions.str_to_scalar_enum(response, enums.AttMode)

	def set_amode(self, amode: enums.AttMode) -> None:
		"""SCPI: OUTPut<HW>:AMODe \n
		Snippet: driver.output.set_amode(amode = enums.AttMode.APASsive) \n
		Switches the mode of the attenuator at the RF output. \n
			:param amode: AUTO| FIXed| APASsive AUTO The attenuator is switched automatically. The level settings are made in the full range. APASsive The attenuator is switched automatically. The level settings are made only for the passive reference circuits. The high-level ranges are not available. FIXed The level settings are made without switching the attenuator. When this operating mode is switched on, the attenuator is fixed to its current position and the resulting variation range is defined.
		"""
		param = Conversions.enum_scalar_to_str(amode, enums.AttMode)
		self._core.io.write(f'OUTPut<HwInstance>:AMODe {param}')

	def clone(self) -> 'OutputCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = OutputCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions
from .....Internal.Utilities import trim_str_response
from ..... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class GpibCls:
	"""Gpib commands group definition. 3 total commands, 1 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("gpib", core, parent)

	@property
	def self(self):
		"""self commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_self'):
			from .Self import SelfCls
			self._self = SelfCls(self._core, self._cmd_group)
		return self._self

	# noinspection PyTypeChecker
	def get_lterminator(self) -> enums.IecTermMode:
		"""SCPI: SYSTem:COMMunicate:GPIB:LTERminator \n
		Snippet: value: enums.IecTermMode = driver.system.communicate.gpib.get_lterminator() \n
		Sets the terminator recognition for remote control via GPIB bus. \n
			:return: lterminator: STANdard| EOI EOI The terminator must be sent together with the line message EOI (End of Line) . This setting is recommended for binary block transmissions where a character could coincidentally have the value LF (Line Feed) but is not intended as the terminator. This setting must be selected for block data with undefined length. STANdard An LF (Line Feed) is recognized as the terminator regardless of whether it is sent with or without EOI.
		"""
		response = self._core.io.query_str('SYSTem:COMMunicate:GPIB:LTERminator?')
		return Conversions.str_to_scalar_enum(response, enums.IecTermMode)

	def set_lterminator(self, lterminator: enums.IecTermMode) -> None:
		"""SCPI: SYSTem:COMMunicate:GPIB:LTERminator \n
		Snippet: driver.system.communicate.gpib.set_lterminator(lterminator = enums.IecTermMode.EOI) \n
		Sets the terminator recognition for remote control via GPIB bus. \n
			:param lterminator: STANdard| EOI EOI The terminator must be sent together with the line message EOI (End of Line) . This setting is recommended for binary block transmissions where a character could coincidentally have the value LF (Line Feed) but is not intended as the terminator. This setting must be selected for block data with undefined length. STANdard An LF (Line Feed) is recognized as the terminator regardless of whether it is sent with or without EOI.
		"""
		param = Conversions.enum_scalar_to_str(lterminator, enums.IecTermMode)
		self._core.io.write(f'SYSTem:COMMunicate:GPIB:LTERminator {param}')

	def get_resource(self) -> str:
		"""SCPI: SYSTem:COMMunicate:GPIB:RESource \n
		Snippet: value: str = driver.system.communicate.gpib.get_resource() \n
		Queries the VISA resource string for remote control via the GPIB interface. To change the GPIB address, use the command
		method RsSgt.System.Communicate.Gpib.Self.address. \n
			:return: resource: string
		"""
		response = self._core.io.query_str('SYSTem:COMMunicate:GPIB:RESource?')
		return trim_str_response(response)

	def clone(self) -> 'GpibCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = GpibCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions
from .... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class ModextCls:
	"""Modext commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("modext", core, parent)

	# noinspection PyTypeChecker
	def get_impedance(self) -> enums.ImpG50G10K:
		"""SCPI: [SOURce]:INPut:MODext:IMPedance \n
		Snippet: value: enums.ImpG50G10K = driver.source.inputPy.modext.get_impedance() \n
		No command help available \n
			:return: impedance: No help available
		"""
		response = self._core.io.query_str('SOURce:INPut:MODext:IMPedance?')
		return Conversions.str_to_scalar_enum(response, enums.ImpG50G10K)

	def set_impedance(self, impedance: enums.ImpG50G10K) -> None:
		"""SCPI: [SOURce]:INPut:MODext:IMPedance \n
		Snippet: driver.source.inputPy.modext.set_impedance(impedance = enums.ImpG50G10K.G10K) \n
		No command help available \n
			:param impedance: No help available
		"""
		param = Conversions.enum_scalar_to_str(impedance, enums.ImpG50G10K)
		self._core.io.write(f'SOURce:INPut:MODext:IMPedance {param}')



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions
from .... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class TriggerCls:
	"""Trigger commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("trigger", core, parent)

	# noinspection PyTypeChecker
	def get_rmode(self) -> enums.TrigRunMode:
		"""SCPI: [SOURce<HW>]:BB:TRIGger:RMODe \n
		Snippet: value: enums.TrigRunMode = driver.source.bb.trigger.get_rmode() \n
		No command help available \n
			:return: running_mode: No help available
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:TRIGger:RMODe?')
		return Conversions.str_to_scalar_enum(response, enums.TrigRunMode)



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class ImpairmentCls:
	"""Impairment commands group definition. 17 total commands, 4 Subgroups, 0 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("impairment", core, parent)

	@property
	def bbmm(self):
		"""bbmm commands group. 1 Sub-classes, 0 commands."""
		if not hasattr(self, '_bbmm'):
			from .Bbmm import BbmmCls
			self._bbmm = BbmmCls(self._core, self._cmd_group)
		return self._bbmm

	@property
	def iqOutput(self):
		"""iqOutput commands group. 6 Sub-classes, 0 commands."""
		if not hasattr(self, '_iqOutput'):
			from .IqOutput import IqOutputCls
			self._iqOutput = IqOutputCls(self._core, self._cmd_group)
		return self._iqOutput

	@property
	def optimization(self):
		"""optimization commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_optimization'):
			from .Optimization import OptimizationCls
			self._optimization = OptimizationCls(self._core, self._cmd_group)
		return self._optimization

	@property
	def rf(self):
		"""rf commands group. 7 Sub-classes, 0 commands."""
		if not hasattr(self, '_rf'):
			from .Rf import RfCls
			self._rf = RfCls(self._core, self._cmd_group)
		return self._rf

	def clone(self) -> 'ImpairmentCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = ImpairmentCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



"""Utilities for string manipulation and string formatting for the user."""

from enum import Flag
from typing import Tuple


class TrimStringMode(Flag):
	"""Trimming mode for strings."""
	white_chars_only = 1
	white_chars_single_quotes = 2
	white_chars_double_quotes = 3
	white_chars_all_quotes = 4


def trim_str_response(text: str, mode=TrimStringMode.white_chars_all_quotes) -> str:
	"""Trims instrument string response.
	In modes white_chars_all_quotes, white_chars_single_quotes, white_chars_double_quotes:
	All the symmetrical leading and trailing quotation marks are trimmed,
	but only if there are none in the remaining text."""
	first_sq_ix = -1
	first_dq_ix = -1
	rem_sq = True if (mode == TrimStringMode.white_chars_all_quotes or mode == TrimStringMode.white_chars_single_quotes) else False
	rem_dq = True if (mode == TrimStringMode.white_chars_all_quotes or mode == TrimStringMode.white_chars_double_quotes) else False

	if not text:
		return text
	text = text.strip()
	if rem_sq and text == "''":
		return ''
	if rem_dq and text == '""':
		return ''
	start_ix = 0
	end_ix: int = len(text) - 1
	if end_ix - 2 < start_ix:
		return text
	if mode is not TrimStringMode.white_chars_only:
		# Loop to cut the outer paired quotation marks
		trimmed = True
		while trimmed:
			trimmed = False
			if rem_sq and text[start_ix] == "'" and text[end_ix] == "'":
				if first_sq_ix < 0:
					first_sq_ix = start_ix
				start_ix += 1
				end_ix -= 1
				trimmed = True
			if end_ix - 2 < start_ix:
				break
			if rem_dq and text[start_ix] == '"' and text[end_ix] == '"':
				if first_dq_ix < 0:
					first_dq_ix = start_ix
				start_ix += 1
				end_ix -= 1
				trimmed = True
			if end_ix - 2 < start_ix:
				break
		if start_ix == 0:
			return text

		final_cut_ix = start_ix
		shortened_text = text[start_ix: -start_ix]
		if first_sq_ix >= 0 and "'" in shortened_text:
			# The cut quotes are also in the shortened string, do not removed the quotes, and set the cutting to start_ix
			final_cut_ix = first_sq_ix
		if first_dq_ix >= 0 and '"' in shortened_text:
			if final_cut_ix > first_dq_ix:
				final_cut_ix = first_dq_ix

		if final_cut_ix == 0:
			return text

		text = text[final_cut_ix: -final_cut_ix]

	return text


def truncate_string_from_end(string: str, max_len: int) -> str:
	"""If the string len is below the max_len, the function returns the same string.
	If the string is above the max len, the function returns only the last max_len characters plus '...' at the beginning."""
	if len(string) <= max_len:
		return string
	return f'Last {max_len} chars: "...{string[-max_len:]}"'


def get_plural_string(word: str, amount: int) -> str:
	"""Returns singular or plural of the word depending on the amount.
	Example:
		word = 'piece', amount = 0 -> '0 pieces'
		word = 'piece', amount = 1 -> '1 piece'
		word = 'piece', amount = 5 -> '5 pieces'"""
	if amount == 1:
		return f'1 {word}'
	else:
		return f'{amount} {word}s'


def parse_token_to_key_and_value(token: str) -> Tuple[str, str]:
	"""Parses entered string to name and value with the delimiter '='.
	If the token is empty: name = None, value = None.
	If the '=' is not found: name = token, value = None.
	name is trimmed for white spaces.
	value is trimmed with trim_str_response()."""
	token = token.strip()
	if not token:
		# noinspection PyTypeChecker
		return None, None
	if '=' in token:
		data = token.split('=')
		name = data[0].strip()
		value = trim_str_response(data[1])
		return name, value

	# noinspection PyTypeChecker
	return token.strip(), None


def size_to_kb_mb_string(data_size: int, as_additional_info: bool = False) -> str:
	"""Returns human-readable string with kilobytes or megabytes depending on the data_size range. \n
		:param data_size: data size in bytes to convert
		:param as_additional_info:
		if True, the dynamic data appear in round bracket after the number in bytes. e.g. '12345678 bytes (11.7 MB)'
		if False, only the dynamic data is returned e.g. '11.7 MB' """
	if data_size < 1024:
		as_additional_info = False
		dynamic = f'{data_size} bytes'
	elif data_size < 1048576:
		dynamic = f'{data_size / 1024:0.1f} kB'
	else:
		dynamic = f'{data_size / 1048576:0.1f} MB'

	if as_additional_info:
		return f'{data_size} bytes ({dynamic})'
	else:
		return dynamic


def calculate_chunks_count(data_size: int, chunk_size: int) -> int:
	"""Returns number of chunks needed to transfer the data_size split to maximum of chunk_size blocks. \n
	:param data_size: total data size
	:param chunk_size: maximum size of one block"""
	return (data_size // chunk_size) + (1 if (data_size % chunk_size) > 0 else 0)


def escape_nonprintable_chars(string: str, encoding: str = 'charmap') -> str:
	"""
	Replace nonprintable characters in string s by its hex representation.
	"""
	if string.isprintable():
		return string
	new_string = ''
	for char in string:
		if char.isprintable():
			new_string += char
		elif char == '\n':
			new_string += r'\n'
		elif char == '\r':
			new_string += r'\r'
		elif char == '\t':
			new_string += r'\t'
		else:
			byte = bytes(char, encoding)
			char = byte.hex()
			new_string += r'\x' + char
	return new_string


def shorten_string_middle(string: str, max_len: int) -> str:
	"""If the length of the string is bigger than the max_len,
	the middle of the string is abbreviated with ' .... ' """
	count = len(string)
	if count <= max_len:
		return string
	half = int((max_len - 6) / 2)
	md = (max_len - 6) % 2
	return string[:half + md] + ' .... ' + string[(count - half):]



from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class DataCls:
	"""Data commands group definition. 5 total commands, 3 Subgroups, 0 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("data", core, parent)

	@property
	def frequency(self):
		"""frequency commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_frequency'):
			from .Frequency import FrequencyCls
			self._frequency = FrequencyCls(self._core, self._cmd_group)
		return self._frequency

	@property
	def power(self):
		"""power commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_power'):
			from .Power import PowerCls
			self._power = PowerCls(self._core, self._cmd_group)
		return self._power

	@property
	def sensor(self):
		"""sensor commands group. 1 Sub-classes, 0 commands."""
		if not hasattr(self, '_sensor'):
			from .Sensor import SensorCls
			self._sensor = SensorCls(self._core, self._cmd_group)
		return self._sensor

	def clone(self) -> 'DataCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = DataCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup
from ......Internal import Conversions
from ......Internal.Utilities import trim_str_response
from ...... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class DataCls:
	"""Data commands group definition. 3 total commands, 1 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("data", core, parent)

	@property
	def pattern(self):
		"""pattern commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_pattern'):
			from .Pattern import PatternCls
			self._pattern = PatternCls(self._core, self._cmd_group)
		return self._pattern

	def get_dselect(self) -> str:
		"""SCPI: [SOURce<HW>]:BB:XMRadio:DATA:DSELect \n
		Snippet: value: str = driver.source.bb.xmRadio.data.get_dselect() \n
		No command help available \n
			:return: dselect: No help available
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:XMRadio:DATA:DSELect?')
		return trim_str_response(response)

	def set_dselect(self, dselect: str) -> None:
		"""SCPI: [SOURce<HW>]:BB:XMRadio:DATA:DSELect \n
		Snippet: driver.source.bb.xmRadio.data.set_dselect(dselect = '1') \n
		No command help available \n
			:param dselect: No help available
		"""
		param = Conversions.value_to_quoted_str(dselect)
		self._core.io.write(f'SOURce<HwInstance>:BB:XMRadio:DATA:DSELect {param}')

	# noinspection PyTypeChecker
	def get_value(self) -> enums.DataSour:
		"""SCPI: [SOURce<HW>]:BB:XMRadio:DATA \n
		Snippet: value: enums.DataSour = driver.source.bb.xmRadio.data.get_value() \n
		No command help available \n
			:return: data: No help available
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:XMRadio:DATA?')
		return Conversions.str_to_scalar_enum(response, enums.DataSour)

	def set_value(self, data: enums.DataSour) -> None:
		"""SCPI: [SOURce<HW>]:BB:XMRadio:DATA \n
		Snippet: driver.source.bb.xmRadio.data.set_value(data = enums.DataSour.DLISt) \n
		No command help available \n
			:param data: No help available
		"""
		param = Conversions.enum_scalar_to_str(data, enums.DataSour)
		self._core.io.write(f'SOURce<HwInstance>:BB:XMRadio:DATA {param}')

	def clone(self) -> 'DataCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = DataCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from .......Internal.Core import Core
from .......Internal.CommandsGroup import CommandsGroup
from .......Internal import Conversions
from ....... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class EnvelopeCls:
	"""Envelope commands group definition. 47 total commands, 7 Subgroups, 12 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("envelope", core, parent)

	@property
	def emf(self):
		"""emf commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_emf'):
			from .Emf import EmfCls
			self._emf = EmfCls(self._core, self._cmd_group)
		return self._emf

	@property
	def pin(self):
		"""pin commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_pin'):
			from .Pin import PinCls
			self._pin = PinCls(self._core, self._cmd_group)
		return self._pin

	@property
	def power(self):
		"""power commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_power'):
			from .Power import PowerCls
			self._power = PowerCls(self._core, self._cmd_group)
		return self._power

	@property
	def shaping(self):
		"""shaping commands group. 6 Sub-classes, 3 commands."""
		if not hasattr(self, '_shaping'):
			from .Shaping import ShapingCls
			self._shaping = ShapingCls(self._core, self._cmd_group)
		return self._shaping

	@property
	def vcc(self):
		"""vcc commands group. 1 Sub-classes, 3 commands."""
		if not hasattr(self, '_vcc'):
			from .Vcc import VccCls
			self._vcc = VccCls(self._core, self._cmd_group)
		return self._vcc

	@property
	def vout(self):
		"""vout commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_vout'):
			from .Vout import VoutCls
			self._vout = VoutCls(self._core, self._cmd_group)
		return self._vout

	@property
	def vpp(self):
		"""vpp commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_vpp'):
			from .Vpp import VppCls
			self._vpp = VppCls(self._core, self._cmd_group)
		return self._vpp

	# noinspection PyTypeChecker
	def get_adaption(self) -> enums.IqOutEnvAdaption:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:ADAPtion \n
		Snippet: value: enums.IqOutEnvAdaption = driver.source.iq.output.analog.envelope.get_adaption() \n
		Defines the envelope voltage adaption mode. \n
			:return: adaption_mode: AUTO| MANual| POWer AUTO = Auto Normalized, POWer = Auto Power, MANual = Manual
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:ADAPtion?')
		return Conversions.str_to_scalar_enum(response, enums.IqOutEnvAdaption)

	def set_adaption(self, adaption_mode: enums.IqOutEnvAdaption) -> None:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:ADAPtion \n
		Snippet: driver.source.iq.output.analog.envelope.set_adaption(adaption_mode = enums.IqOutEnvAdaption.AUTO) \n
		Defines the envelope voltage adaption mode. \n
			:param adaption_mode: AUTO| MANual| POWer AUTO = Auto Normalized, POWer = Auto Power, MANual = Manual
		"""
		param = Conversions.enum_scalar_to_str(adaption_mode, enums.IqOutEnvAdaption)
		self._core.io.write(f'SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:ADAPtion {param}')

	def get_bias(self) -> float:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:BIAS \n
		Snippet: value: float = driver.source.iq.output.analog.envelope.get_bias() \n
		Sets a bias. \n
			:return: bias: float Range: -3.6V to 3.6V , Unit: V
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:BIAS?')
		return Conversions.str_to_float(response)

	def set_bias(self, bias: float) -> None:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:BIAS \n
		Snippet: driver.source.iq.output.analog.envelope.set_bias(bias = 1.0) \n
		Sets a bias. \n
			:param bias: float Range: -3.6V to 3.6V , Unit: V
		"""
		param = Conversions.decimal_value_to_str(bias)
		self._core.io.write(f'SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:BIAS {param}')

	def get_binput(self) -> bool:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:BINPut \n
		Snippet: value: bool = driver.source.iq.output.analog.envelope.get_binput() \n
		Enables the generation of a bipolar signal. \n
			:return: bipolar_input: 1| ON| 0| OFF
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:BINPut?')
		return Conversions.str_to_bool(response)

	def set_binput(self, bipolar_input: bool) -> None:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:BINPut \n
		Snippet: driver.source.iq.output.analog.envelope.set_binput(bipolar_input = False) \n
		Enables the generation of a bipolar signal. \n
			:param bipolar_input: 1| ON| 0| OFF
		"""
		param = Conversions.bool_to_str(bipolar_input)
		self._core.io.write(f'SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:BINPut {param}')

	def get_delay(self) -> float:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:DELay \n
		Snippet: value: float = driver.source.iq.output.analog.envelope.get_delay() \n
		Enables a time delay of the generated envelope signal relative to the corresponding RF signal. \n
			:return: delay: float Range: -500E-9 to 500E-9
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:DELay?')
		return Conversions.str_to_float(response)

	def set_delay(self, delay: float) -> None:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:DELay \n
		Snippet: driver.source.iq.output.analog.envelope.set_delay(delay = 1.0) \n
		Enables a time delay of the generated envelope signal relative to the corresponding RF signal. \n
			:param delay: float Range: -500E-9 to 500E-9
		"""
		param = Conversions.decimal_value_to_str(delay)
		self._core.io.write(f'SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:DELay {param}')

	# noinspection PyTypeChecker
	def get_etrak(self) -> enums.IqOutEnvEtRak:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:ETRak \n
		Snippet: value: enums.IqOutEnvEtRak = driver.source.iq.output.analog.envelope.get_etrak() \n
		Selects one of the predefined interface types or allows user-defined settings. See Table 'Default parameters per eTrak®
		Interface Type'. \n
			:return: etrak_ifc_type: USER| ET1V2| ET1V5| ET2V0
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:ETRak?')
		return Conversions.str_to_scalar_enum(response, enums.IqOutEnvEtRak)

	def set_etrak(self, etrak_ifc_type: enums.IqOutEnvEtRak) -> None:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:ETRak \n
		Snippet: driver.source.iq.output.analog.envelope.set_etrak(etrak_ifc_type = enums.IqOutEnvEtRak.ET1V2) \n
		Selects one of the predefined interface types or allows user-defined settings. See Table 'Default parameters per eTrak®
		Interface Type'. \n
			:param etrak_ifc_type: USER| ET1V2| ET1V5| ET2V0
		"""
		param = Conversions.enum_scalar_to_str(etrak_ifc_type, enums.IqOutEnvEtRak)
		self._core.io.write(f'SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:ETRak {param}')

	def get_fdpd(self) -> bool:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:FDPD \n
		Snippet: value: bool = driver.source.iq.output.analog.envelope.get_fdpd() \n
		Enables calculation of the envelope from predistorted signal. \n
			:return: calc_from_dpd_stat: 1| ON| 0| OFF
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:FDPD?')
		return Conversions.str_to_bool(response)

	def set_fdpd(self, calc_from_dpd_stat: bool) -> None:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:FDPD \n
		Snippet: driver.source.iq.output.analog.envelope.set_fdpd(calc_from_dpd_stat = False) \n
		Enables calculation of the envelope from predistorted signal. \n
			:param calc_from_dpd_stat: 1| ON| 0| OFF
		"""
		param = Conversions.bool_to_str(calc_from_dpd_stat)
		self._core.io.write(f'SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:FDPD {param}')

	def get_gain(self) -> float:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:GAIN \n
		Snippet: value: float = driver.source.iq.output.analog.envelope.get_gain() \n
		Sets the gain of the used external DC modulator. \n
			:return: gain: float Range: -50 to 50
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:GAIN?')
		return Conversions.str_to_float(response)

	def set_gain(self, gain: float) -> None:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:GAIN \n
		Snippet: driver.source.iq.output.analog.envelope.set_gain(gain = 1.0) \n
		Sets the gain of the used external DC modulator. \n
			:param gain: float Range: -50 to 50
		"""
		param = Conversions.decimal_value_to_str(gain)
		self._core.io.write(f'SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:GAIN {param}')

	def get_offset(self) -> float:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:OFFSet \n
		Snippet: value: float = driver.source.iq.output.analog.envelope.get_offset() \n
		Sets an offset between the envelope and the inverted envelope signal. \n
			:return: offset: float Range: -8V to 8V , Unit: V
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:OFFSet?')
		return Conversions.str_to_float(response)

	def set_offset(self, offset: float) -> None:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:OFFSet \n
		Snippet: driver.source.iq.output.analog.envelope.set_offset(offset = 1.0) \n
		Sets an offset between the envelope and the inverted envelope signal. \n
			:param offset: float Range: -8V to 8V , Unit: V
		"""
		param = Conversions.decimal_value_to_str(offset)
		self._core.io.write(f'SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:OFFSet {param}')

	def get_rin(self) -> float:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:RIN \n
		Snippet: value: float = driver.source.iq.output.analog.envelope.get_rin() \n
		Sets the input impedance Rin of the used external DC modulator. \n
			:return: ipart_nput_resistance: No help available
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:RIN?')
		return Conversions.str_to_float(response)

	def set_rin(self, ipart_nput_resistance: float) -> None:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:RIN \n
		Snippet: driver.source.iq.output.analog.envelope.set_rin(ipart_nput_resistance = 1.0) \n
		Sets the input impedance Rin of the used external DC modulator. \n
			:param ipart_nput_resistance: float Range: 50|100 to 1E6
		"""
		param = Conversions.decimal_value_to_str(ipart_nput_resistance)
		self._core.io.write(f'SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:RIN {param}')

	def get_state(self) -> bool:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:STATe \n
		Snippet: value: bool = driver.source.iq.output.analog.envelope.get_state() \n
		Enables the output of a control signal that follows the RF envelope. \n
			:return: state: 1| ON| 0| OFF
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:STATe?')
		return Conversions.str_to_bool(response)

	def set_state(self, state: bool) -> None:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:STATe \n
		Snippet: driver.source.iq.output.analog.envelope.set_state(state = False) \n
		Enables the output of a control signal that follows the RF envelope. \n
			:param state: 1| ON| 0| OFF
		"""
		param = Conversions.bool_to_str(state)
		self._core.io.write(f'SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:STATe {param}')

	# noinspection PyTypeChecker
	def get_termination(self) -> enums.IqOutEnvTerm:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:TERMination \n
		Snippet: value: enums.IqOutEnvTerm = driver.source.iq.output.analog.envelope.get_termination() \n
		Sets how the inputs of the DC modulator are terminated. \n
			:return: termination: GROund| WIRE
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:TERMination?')
		return Conversions.str_to_scalar_enum(response, enums.IqOutEnvTerm)

	def set_termination(self, termination: enums.IqOutEnvTerm) -> None:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:TERMination \n
		Snippet: driver.source.iq.output.analog.envelope.set_termination(termination = enums.IqOutEnvTerm.GROund) \n
		Sets how the inputs of the DC modulator are terminated. \n
			:param termination: GROund| WIRE
		"""
		param = Conversions.enum_scalar_to_str(termination, enums.IqOutEnvTerm)
		self._core.io.write(f'SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:TERMination {param}')

	# noinspection PyTypeChecker
	def get_vref(self) -> enums.IqOutEnvVrEf:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:VREF \n
		Snippet: value: enums.IqOutEnvVrEf = driver.source.iq.output.analog.envelope.get_vref() \n
		Defines whether the envelope voltage Vout is set directly or it is estimated from the selected supply voltage Vcc. \n
			:return: voltage_reference: VCC| VOUT
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:VREF?')
		return Conversions.str_to_scalar_enum(response, enums.IqOutEnvVrEf)

	def set_vref(self, voltage_reference: enums.IqOutEnvVrEf) -> None:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:VREF \n
		Snippet: driver.source.iq.output.analog.envelope.set_vref(voltage_reference = enums.IqOutEnvVrEf.VCC) \n
		Defines whether the envelope voltage Vout is set directly or it is estimated from the selected supply voltage Vcc. \n
			:param voltage_reference: VCC| VOUT
		"""
		param = Conversions.enum_scalar_to_str(voltage_reference, enums.IqOutEnvVrEf)
		self._core.io.write(f'SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:VREF {param}')

	def clone(self) -> 'EnvelopeCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = EnvelopeCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from enum import Enum
# noinspection PyPep8Naming
from .Internal.RepeatedCapability import VALUE_DEFAULT as DefaultRepCap
# noinspection PyPep8Naming
from .Internal.RepeatedCapability import VALUE_EMPTY as EmptyRepCap


# noinspection SpellCheckingInspection
class HwInstance(Enum):
	"""Global Repeated capability HwInstance"""
	Empty = EmptyRepCap
	Default = DefaultRepCap
	InstA = 1
	InstB = 2
	InstC = 3
	InstD = 4
	InstE = 5
	InstF = 6
	InstG = 7
	InstH = 8


# noinspection SpellCheckingInspection
class BitNumberNull(Enum):
	"""Repeated capability BitNumberNull"""
	Empty = EmptyRepCap
	Default = DefaultRepCap
	Nr0 = 0
	Nr1 = 1
	Nr2 = 2
	Nr3 = 3
	Nr4 = 4
	Nr5 = 5
	Nr6 = 6
	Nr7 = 7
	Nr8 = 8
	Nr9 = 9
	Nr10 = 10
	Nr11 = 11
	Nr12 = 12
	Nr13 = 13
	Nr14 = 14
	Nr15 = 15


# noinspection SpellCheckingInspection
class Carrier(Enum):
	"""Repeated capability Carrier"""
	Empty = EmptyRepCap
	Default = DefaultRepCap
	Nr1 = 1
	Nr2 = 2
	Nr3 = 3
	Nr4 = 4
	Nr5 = 5
	Nr6 = 6
	Nr7 = 7
	Nr8 = 8
	Nr9 = 9
	Nr10 = 10
	Nr11 = 11
	Nr12 = 12
	Nr13 = 13
	Nr14 = 14
	Nr15 = 15
	Nr16 = 16
	Nr17 = 17
	Nr18 = 18
	Nr19 = 19
	Nr20 = 20
	Nr21 = 21
	Nr22 = 22
	Nr23 = 23
	Nr24 = 24
	Nr25 = 25
	Nr26 = 26
	Nr27 = 27
	Nr28 = 28
	Nr29 = 29
	Nr30 = 30
	Nr31 = 31
	Nr32 = 32
	Nr33 = 33
	Nr34 = 34
	Nr35 = 35
	Nr36 = 36
	Nr37 = 37
	Nr38 = 38
	Nr39 = 39
	Nr40 = 40
	Nr41 = 41
	Nr42 = 42
	Nr43 = 43
	Nr44 = 44
	Nr45 = 45
	Nr46 = 46
	Nr47 = 47
	Nr48 = 48
	Nr49 = 49
	Nr50 = 50
	Nr51 = 51
	Nr52 = 52
	Nr53 = 53
	Nr54 = 54
	Nr55 = 55
	Nr56 = 56
	Nr57 = 57
	Nr58 = 58
	Nr59 = 59
	Nr60 = 60
	Nr61 = 61
	Nr62 = 62
	Nr63 = 63
	Nr64 = 64


# noinspection SpellCheckingInspection
class Channel(Enum):
	"""Repeated capability Channel"""
	Empty = EmptyRepCap
	Default = DefaultRepCap
	Nr1 = 1
	Nr2 = 2
	Nr3 = 3
	Nr4 = 4
	Nr5 = 5
	Nr6 = 6
	Nr7 = 7
	Nr8 = 8
	Nr9 = 9
	Nr10 = 10
	Nr11 = 11
	Nr12 = 12
	Nr13 = 13
	Nr14 = 14
	Nr15 = 15
	Nr16 = 16
	Nr17 = 17
	Nr18 = 18
	Nr19 = 19
	Nr20 = 20
	Nr21 = 21
	Nr22 = 22
	Nr23 = 23
	Nr24 = 24
	Nr25 = 25
	Nr26 = 26
	Nr27 = 27
	Nr28 = 28
	Nr29 = 29
	Nr30 = 30
	Nr31 = 31
	Nr32 = 32
	Nr33 = 33
	Nr34 = 34
	Nr35 = 35
	Nr36 = 36
	Nr37 = 37
	Nr38 = 38
	Nr39 = 39
	Nr40 = 40
	Nr41 = 41
	Nr42 = 42
	Nr43 = 43
	Nr44 = 44
	Nr45 = 45
	Nr46 = 46
	Nr47 = 47
	Nr48 = 48
	Nr49 = 49
	Nr50 = 50
	Nr51 = 51
	Nr52 = 52
	Nr53 = 53
	Nr54 = 54
	Nr55 = 55
	Nr56 = 56
	Nr57 = 57
	Nr58 = 58
	Nr59 = 59
	Nr60 = 60
	Nr61 = 61
	Nr62 = 62
	Nr63 = 63
	Nr64 = 64


# noinspection SpellCheckingInspection
class ErrorCount(Enum):
	"""Repeated capability ErrorCount"""
	Empty = EmptyRepCap
	Default = DefaultRepCap
	Nr1 = 1
	Nr2 = 2
	Nr3 = 3
	Nr4 = 4
	Nr5 = 5
	Nr6 = 6
	Nr7 = 7
	Nr8 = 8
	Nr9 = 9
	Nr10 = 10
	Nr11 = 11
	Nr12 = 12
	Nr13 = 13
	Nr14 = 14
	Nr15 = 15
	Nr16 = 16


# noinspection SpellCheckingInspection
class External(Enum):
	"""Repeated capability External"""
	Empty = EmptyRepCap
	Default = DefaultRepCap
	Nr1 = 1
	Nr2 = 2
	Nr3 = 3
	Nr4 = 4


# noinspection SpellCheckingInspection
class IqConnector(Enum):
	"""Repeated capability IqConnector"""
	Empty = EmptyRepCap
	Default = DefaultRepCap
	Nr1 = 1
	Nr2 = 2
	Nr3 = 3
	Nr4 = 4


# noinspection SpellCheckingInspection
class Level(Enum):
	"""Repeated capability Level"""
	Empty = EmptyRepCap
	Default = DefaultRepCap
	Nr1 = 1
	Nr2 = 2
	Nr3 = 3
	Nr4 = 4
	Nr5 = 5
	Nr6 = 6
	Nr7 = 7
	Nr8 = 8
	Nr9 = 9
	Nr10 = 10
	Nr11 = 11
	Nr12 = 12
	Nr13 = 13
	Nr14 = 14
	Nr15 = 15
	Nr16 = 16


# noinspection SpellCheckingInspection
class Output(Enum):
	"""Repeated capability Output"""
	Empty = EmptyRepCap
	Default = DefaultRepCap
	Nr1 = 1
	Nr2 = 2
	Nr3 = 3
	Nr4 = 4
	Nr5 = 5
	Nr6 = 6
	Nr7 = 7
	Nr8 = 8
	Nr9 = 9
	Nr10 = 10
	Nr11 = 11
	Nr12 = 12
	Nr13 = 13
	Nr14 = 14
	Nr15 = 15
	Nr16 = 16
	Nr17 = 17
	Nr18 = 18
	Nr19 = 19
	Nr20 = 20
	Nr21 = 21
	Nr22 = 22
	Nr23 = 23
	Nr24 = 24
	Nr25 = 25
	Nr26 = 26
	Nr27 = 27
	Nr28 = 28
	Nr29 = 29
	Nr30 = 30
	Nr31 = 31
	Nr32 = 32
	Nr33 = 33
	Nr34 = 34
	Nr35 = 35
	Nr36 = 36
	Nr37 = 37
	Nr38 = 38
	Nr39 = 39
	Nr40 = 40
	Nr41 = 41
	Nr42 = 42
	Nr43 = 43
	Nr44 = 44
	Nr45 = 45
	Nr46 = 46
	Nr47 = 47
	Nr48 = 48
	Nr49 = 49
	Nr50 = 50
	Nr51 = 51
	Nr52 = 52
	Nr53 = 53
	Nr54 = 54
	Nr55 = 55
	Nr56 = 56
	Nr57 = 57
	Nr58 = 58
	Nr59 = 59
	Nr60 = 60
	Nr61 = 61
	Nr62 = 62
	Nr63 = 63
	Nr64 = 64


# noinspection SpellCheckingInspection
class Path(Enum):
	"""Repeated capability Path"""
	Empty = EmptyRepCap
	Default = DefaultRepCap
	Nr1 = 1
	Nr2 = 2
	Nr3 = 3
	Nr4 = 4
	Nr5 = 5
	Nr6 = 6
	Nr7 = 7
	Nr8 = 8


# noinspection SpellCheckingInspection
class UserIx(Enum):
	"""Repeated capability UserIx"""
	Empty = EmptyRepCap
	Default = DefaultRepCap
	Nr1 = 1
	Nr2 = 2
	Nr3 = 3
	Nr4 = 4
	Nr5 = 5
	Nr6 = 6
	Nr7 = 7
	Nr8 = 8
	Nr9 = 9
	Nr10 = 10
	Nr11 = 11
	Nr12 = 12
	Nr13 = 13
	Nr14 = 14
	Nr15 = 15
	Nr16 = 16
	Nr17 = 17
	Nr18 = 18
	Nr19 = 19
	Nr20 = 20
	Nr21 = 21
	Nr22 = 22
	Nr23 = 23
	Nr24 = 24
	Nr25 = 25
	Nr26 = 26
	Nr27 = 27
	Nr28 = 28
	Nr29 = 29
	Nr30 = 30
	Nr31 = 31
	Nr32 = 32
	Nr33 = 33
	Nr34 = 34
	Nr35 = 35
	Nr36 = 36
	Nr37 = 37
	Nr38 = 38
	Nr39 = 39
	Nr40 = 40
	Nr41 = 41
	Nr42 = 42
	Nr43 = 43
	Nr44 = 44
	Nr45 = 45
	Nr46 = 46
	Nr47 = 47
	Nr48 = 48



from .......Internal.Core import Core
from .......Internal.CommandsGroup import CommandsGroup
from .......Internal import Conversions
from ....... import repcap


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class FrequencyCls:
	"""Frequency commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("frequency", core, parent)

	def set(self, frequency: int, carrier=repcap.Carrier.Default) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:CARRier<CH>:FREQuency \n
		Snippet: driver.source.bb.arbitrary.mcarrier.carrier.frequency.set(frequency = 1, carrier = repcap.Carrier.Default) \n
		Sets or indicates the carrier frequency, depending on the selected carrier frequency mode. The carrier frequency can be
		set in 'Arbitrary Carrier frequency' mode. For 'Equidistant Carrier Spacing', the carrier spacing is determined
		automatically. \n
			:param frequency: float Value range depends on the max bandwidth, see data sheet Range: -40E6 to 40E6
			:param carrier: optional repeated capability selector. Default value: Nr1 (settable in the interface 'Mcarrier')
		"""
		param = Conversions.decimal_value_to_str(frequency)
		carrier_cmd_val = self._cmd_group.get_repcap_cmd_value(carrier, repcap.Carrier)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:MCARrier:CARRier{carrier_cmd_val}:FREQuency {param}')

	def get(self, carrier=repcap.Carrier.Default) -> int:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:CARRier<CH>:FREQuency \n
		Snippet: value: int = driver.source.bb.arbitrary.mcarrier.carrier.frequency.get(carrier = repcap.Carrier.Default) \n
		Sets or indicates the carrier frequency, depending on the selected carrier frequency mode. The carrier frequency can be
		set in 'Arbitrary Carrier frequency' mode. For 'Equidistant Carrier Spacing', the carrier spacing is determined
		automatically. \n
			:param carrier: optional repeated capability selector. Default value: Nr1 (settable in the interface 'Mcarrier')
			:return: frequency: float Value range depends on the max bandwidth, see data sheet Range: -40E6 to 40E6"""
		carrier_cmd_val = self._cmd_group.get_repcap_cmd_value(carrier, repcap.Carrier)
		response = self._core.io.query_str(f'SOURce<HwInstance>:BB:ARBitrary:MCARrier:CARRier{carrier_cmd_val}:FREQuency?')
		return Conversions.str_to_int(response)



"""Provides:
- class SingleComposer - for composing SCPI string parameters from ArgSingle
- class StructComposer - for composing SCPI string parameters from ArgStruct
- function compose_cmd_string_from_struct_args - takes ArgStruct values and converts it to a SCPI string parameter.
- function compose_cmd_string_from_single_args - takes ArgSingle[] values and converts them to a SCPI string parameter.
The composing of the SCPI parameter string is similar for the ArgStruct and ArgSingle[] objects, therefore they share the same module."""

from typing import Dict

from .ArgSingle import ArgSingle
from .ArgStruct import ArgStruct
from .Utilities import get_plural_string
from .InstrumentErrors import RsInstrException


class SingleComposer:
	"""Composes strings for single argument.
	Provides Composer interface with 3 functions:
	- from_scalar_arg
	- from_list_arg
	- get_arg_list_size"""

	@staticmethod
	def from_scalar_arg(arg: ArgSingle) -> str:
		"""Takes argument's scalar value and converts it to a SCPI parameter string."""
		if not arg.is_optional and arg.value is None:
			raise ValueError(f"StructArgComposer.from_scalar_arg() - mandatory argument's '{arg.name}' value is None")
		assert arg.data_type.is_scalar, f"StructArgComposer.from_scalar_arg() - argument '{arg.name}' must be scalar. Data type: '{arg.data_type}'"
		if arg.value is None:
			return ''
		return arg.conv_to_scpi_string.get_value(arg.value)

	@staticmethod
	def from_list_arg(arg: ArgSingle, start_ix=0, items_count=-1) -> str:
		"""Takes argument's List of elements and converts it to a csv-string of SCPI parameters.
		start_ix defines where to start in the list.
		items_count defines how many items to take. -1 means all of them"""
		if not arg.is_optional and arg.value is None:
			raise ValueError(f"StructArgComposer.from_list_arg() - mandatory argument's '{arg.name}' value is None")
		assert arg.data_type.is_list, f"StructArgComposer.from_list_arg() - argument must be list. Data type: '{arg.data_type}'"
		if not arg.is_optional:
			assert arg.value is not None, f"StructArgComposer.from_list_arg() - mandatory argument's '{arg.name}' value is None"
		if arg.value is None:
			return ''
		if start_ix == 0 and items_count < 0:
			return arg.conv_to_scpi_string.get_value(arg.value)
		elif items_count < 0:
			return arg.conv_to_scpi_string.get_value(arg.value[start_ix])
		elif items_count == 0:
			return ''
		elif items_count > 0:
			return arg.conv_to_scpi_string.get_value(arg.value[start_ix: start_ix + items_count])

	@staticmethod
	def get_arg_list_size(arg: ArgSingle) -> int:
		"""Returns list size of the argument assuming it is a list.
		If the argument is not a list, the method returns -1."""
		if arg.data_type.is_scalar:
			return -1
		else:
			return len(arg.value)


class StructComposer:
	"""Composes strings for structure arguments.
	Provides Composer interface with 3 functions:
	- from_scalar_arg
	- from_list_arg
	- get_arg_list_size \n
	The StructComposer constructor has the owning struct instance as parameter, because this is needed to access the argument value."""

	def __init__(self, struct):
		self.struct = struct

	def from_scalar_arg(self, arg: ArgStruct) -> str:
		"""Returns a single argument value converted to string."""
		assert arg.data_type.is_scalar, f"StructArgComposer.from_scalar_arg() - argument must be scalar. Data type: '{arg.data_type}'"
		return arg.conv_to_scpi_string.get_value(getattr(self.struct, arg.name))

	def from_list_arg(self, arg: ArgStruct, start_ix=0, items_count=-1) -> str:
		"""Returns csv-string with all the elements from the argument value."""
		assert arg.data_type.is_list, f"StructArgComposer.from_list_arg() - argument must be list. Data type: '{arg.data_type}'"
		if start_ix == 0 and items_count < 0:
			return arg.conv_to_scpi_string.get_value(getattr(self.struct, arg.name))
		elif items_count < 0:
			return arg.conv_to_scpi_string.get_value(getattr(self.struct, arg.name)[start_ix])
		elif items_count == 0:
			return ''
		elif items_count > 0:
			return arg.conv_to_scpi_string.get_value(getattr(self.struct, arg.name)[start_ix: start_ix + items_count])

	def get_arg_list_size(self, arg: ArgStruct) -> int:
		"""Returns list size of the argument assuming it is a list.
		If the argument is not a list, the method returns -1."""
		if arg.data_type.is_scalar:
			return -1
		else:
			return len(getattr(self.struct, arg.name))


def compose_cmd_string_from_struct_args(args: Dict[int, ArgStruct], composer: StructComposer, values_obj: object) -> str:
	"""Returns SCPI-composed string based on the structure args specification."""
	arg_count = len(args)
	string_arg = []
	arg_ix = 0
	opt_null_ix = -1

	# Non-repeated arguments - single values or arrays which are not open lists
	# Non-open lists or scalars must be at the beginning of the definition
	# Once the first open list argument is detected, continue further
	while arg_ix < arg_count and args[arg_ix].is_open_list is False:
		arg = args[arg_ix]
		if arg.has_value(values_obj) is False:
			arg.assert_is_optional(values_obj)
			# The optional argument, which has no value. End the entire string_arg composition
			opt_null_ix = arg_ix
			break

		if arg.repetition <= 1:
			# No list, but scalar value
			string_arg.append(composer.from_scalar_arg(arg))
		else:
			string_arg.append(composer.from_list_arg(arg, 0, arg.repetition))

		arg_ix += 1
	# End of non-repeated arguments

	# If the optional argument without a value was not found, check if there are some more arguments to go through
	if opt_null_ix < 0 and arg_ix < arg_count:
		# Still some args to go
		arg = args[arg_ix]
		if arg.is_open_list:
			# The previous loop ended because the next argument had is_open_list = True
			if arg_ix == (arg_count - 1):
				if arg.has_value(values_obj):
					# The last argument, ignore the repetitions and convert the whole list to string
					string_arg.append(composer.from_list_arg(arg))
				else:
					# The optional argument, which has no value. End the entire string_arg composition
					arg.assert_is_optional(values_obj)
					opt_null_ix = arg_ix
			else:
				# More than one argument remaining. Loop through them interleaving the result strings
				# Interleaving arguments must all have values

				# Check if each list has at least Repetition number of elements
				cycles_error = False
				alignments_error = False
				cycle = -1
				data = {}
				for x in range(arg_ix, arg_count):
					arg = args[x]
					curr_size: int = composer.get_arg_list_size(arg)
					curr_cycle: int = curr_size // arg.repetition
					curr_align: int = curr_size % arg.repetition
					data[x] = (curr_size, curr_cycle, curr_align)

					if curr_size < 0:
						raise RsInstrException(f"Argument '{arg.name}' has repetitions, therefore it must be declared as a list. Current Declaration: '{arg.data_type}'")

					if arg.repetition > curr_size:
						raise RsInstrException(f"Argument '{arg.name}' has repetitions {arg.repetition}, but its list size is only {curr_size}")

					# noinspection PyChainedComparisons
					if cycle >= 0 and curr_cycle != cycle:
						cycles_error = True

					cycle = curr_cycle

					if curr_align != 0:
						alignments_error = True

				if cycles_error:
					message = 'Arguments interleaving is not aligned - all the cycles must be the same. Actual cycles:\n'
					for x in range(arg_ix, arg_count):
						message += f'{args[x].name}[{data[x][0]}] sliced by {get_plural_string("element", args[x].repetition)} ' \
							f'results in {data[x][0] / args[x].repetition} cycles\n'
					raise RsInstrException(message)

				if alignments_error:
					message = 'At least one argument has a list size not dividable by the defined repetitions:\n'
					for x in range(arg_ix, arg_count):
						message += f'{args[x].name}[{data[x][0]}] modulo {args[x].repetition}x results in {data[x][0] % args[x].repetition}\n'
					raise RsInstrException(message)

				for x in range(cycle):
					for y in range(arg_ix, arg_count):
						arg = args[y]
						string_arg.append(composer.from_list_arg(arg, arg.repetition * x, arg.repetition))

	if opt_null_ix >= 0:
		# Check the rest of the optional values, all of them must be without a value
		rest = []
		for y in range(opt_null_ix, arg_count):
			arg = args[y]
			if arg.has_value(values_obj):
				rest.append(arg.name)

		if len(rest):
			msg = f"Optional Argument '{args[opt_null_ix].name}' has no value, but the further ones do. " \
				f"If you skip an optional argument, you have to skip all the ones following it. " \
				f"Clear the values for the rest of the argument(s):\n{', '.join(rest)}"
			raise RsInstrException(msg)

	return ','.join(string_arg)


def compose_cmd_string_from_single_args(args: Dict[int, ArgSingle]) -> str:
	"""Returns SCPI-composed string based on the single args specification.
	We can use the same function as for the struct arguments, with the difference of providing a SingleComposer.
	Parameter value_obj is set to None, since each argument holds the value itself."""
	# noinspection PyTypeChecker
	return compose_cmd_string_from_struct_args(args, SingleComposer(), None)



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions
from ....Internal.Utilities import trim_str_response


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class HwAccessCls:
	"""HwAccess commands group definition. 3 total commands, 0 Subgroups, 3 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("hwAccess", core, parent)

	def get_description(self) -> str:
		"""SCPI: SYSTem:PROFiling:HWACcess:DESCription \n
		Snippet: value: str = driver.system.profiling.hwAccess.get_description() \n
		No command help available \n
			:return: description: No help available
		"""
		response = self._core.io.query_str('SYSTem:PROFiling:HWACcess:DESCription?')
		return trim_str_response(response)

	def get_pduration(self) -> int:
		"""SCPI: SYSTem:PROFiling:HWACcess:PDURation \n
		Snippet: value: int = driver.system.profiling.hwAccess.get_pduration() \n
		No command help available \n
			:return: duration_us: No help available
		"""
		response = self._core.io.query_str('SYSTem:PROFiling:HWACcess:PDURation?')
		return Conversions.str_to_int(response)

	def set_pduration(self, duration_us: int) -> None:
		"""SCPI: SYSTem:PROFiling:HWACcess:PDURation \n
		Snippet: driver.system.profiling.hwAccess.set_pduration(duration_us = 1) \n
		No command help available \n
			:param duration_us: No help available
		"""
		param = Conversions.decimal_value_to_str(duration_us)
		self._core.io.write(f'SYSTem:PROFiling:HWACcess:PDURation {param}')

	def get_state(self) -> bool:
		"""SCPI: SYSTem:PROFiling:HWACcess:STATe \n
		Snippet: value: bool = driver.system.profiling.hwAccess.get_state() \n
		No command help available \n
			:return: state: No help available
		"""
		response = self._core.io.query_str('SYSTem:PROFiling:HWACcess:STATe?')
		return Conversions.str_to_bool(response)

	def set_state(self, state: bool) -> None:
		"""SCPI: SYSTem:PROFiling:HWACcess:STATe \n
		Snippet: driver.system.profiling.hwAccess.set_state(state = False) \n
		No command help available \n
			:param state: No help available
		"""
		param = Conversions.bool_to_str(state)
		self._core.io.write(f'SYSTem:PROFiling:HWACcess:STATe {param}')



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions
from ..... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class SensorCls:
	"""Sensor commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("sensor", core, parent)

	def get_aperture(self) -> float:
		"""SCPI: [SOURce<HW>]:POWer:SERVoing:SENSor:APERture \n
		Snippet: value: float = driver.source.power.servoing.sensor.get_aperture() \n
		Sets the aperture time (size of the acquisition interval) of the power sensor during power servoing. \n
			:return: aperture: float Range: 10e-6 to 100e-3
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:POWer:SERVoing:SENSor:APERture?')
		return Conversions.str_to_float(response)

	def set_aperture(self, aperture: float) -> None:
		"""SCPI: [SOURce<HW>]:POWer:SERVoing:SENSor:APERture \n
		Snippet: driver.source.power.servoing.sensor.set_aperture(aperture = 1.0) \n
		Sets the aperture time (size of the acquisition interval) of the power sensor during power servoing. \n
			:param aperture: float Range: 10e-6 to 100e-3
		"""
		param = Conversions.decimal_value_to_str(aperture)
		self._core.io.write(f'SOURce<HwInstance>:POWer:SERVoing:SENSor:APERture {param}')

	# noinspection PyTypeChecker
	def get_value(self) -> enums.PowSensWithUndef:
		"""SCPI: [SOURce<HW>]:POWer:SERVoing:SENSor \n
		Snippet: value: enums.PowSensWithUndef = driver.source.power.servoing.sensor.get_value() \n
		Sets the power sensor as mapped with the remote command SLISt:ELEMent<ch>:MAPPing. \n
			:return: sensor: SENS1| SENS2| SENSor2| SENS3| SENSor3| SENS4| SENSor4| SENSor1| UNDefined
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:POWer:SERVoing:SENSor?')
		return Conversions.str_to_scalar_enum(response, enums.PowSensWithUndef)

	def set_value(self, sensor: enums.PowSensWithUndef) -> None:
		"""SCPI: [SOURce<HW>]:POWer:SERVoing:SENSor \n
		Snippet: driver.source.power.servoing.sensor.set_value(sensor = enums.PowSensWithUndef.SENS1) \n
		Sets the power sensor as mapped with the remote command SLISt:ELEMent<ch>:MAPPing. \n
			:param sensor: SENS1| SENS2| SENSor2| SENS3| SENSor3| SENS4| SENSor4| SENSor1| UNDefined
		"""
		param = Conversions.enum_scalar_to_str(sensor, enums.PowSensWithUndef)
		self._core.io.write(f'SOURce<HwInstance>:POWer:SERVoing:SENSor {param}')



from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup
from ...Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class UpdateCls:
	"""Update commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("update", core, parent)

	def get_hold(self) -> bool:
		"""SCPI: DISPlay:UPDate:HOLD \n
		Snippet: value: bool = driver.display.update.get_hold() \n
		No command help available \n
			:return: hold: No help available
		"""
		response = self._core.io.query_str('DISPlay:UPDate:HOLD?')
		return Conversions.str_to_bool(response)

	def set_hold(self, hold: bool) -> None:
		"""SCPI: DISPlay:UPDate:HOLD \n
		Snippet: driver.display.update.set_hold(hold = False) \n
		No command help available \n
			:param hold: No help available
		"""
		param = Conversions.bool_to_str(hold)
		self._core.io.write(f'DISPlay:UPDate:HOLD {param}')



from ........Internal.Core import Core
from ........Internal.CommandsGroup import CommandsGroup
from ........Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class SynchronizeCls:
	"""Synchronize commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("synchronize", core, parent)

	def get_output(self) -> bool:
		"""SCPI: [SOURce<HW>]:BB:XMRadio:SATellite:TRIGger:EXTernal:SYNChronize:OUTPut \n
		Snippet: value: bool = driver.source.bb.xmRadio.satellite.trigger.external.synchronize.get_output() \n
		No command help available \n
			:return: output: No help available
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:XMRadio:SATellite:TRIGger:EXTernal:SYNChronize:OUTPut?')
		return Conversions.str_to_bool(response)

	def set_output(self, output: bool) -> None:
		"""SCPI: [SOURce<HW>]:BB:XMRadio:SATellite:TRIGger:EXTernal:SYNChronize:OUTPut \n
		Snippet: driver.source.bb.xmRadio.satellite.trigger.external.synchronize.set_output(output = False) \n
		No command help available \n
			:param output: No help available
		"""
		param = Conversions.bool_to_str(output)
		self._core.io.write(f'SOURce<HwInstance>:BB:XMRadio:SATellite:TRIGger:EXTernal:SYNChronize:OUTPut {param}')



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class PowerCls:
	"""Power commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("power", core, parent)

	def get_peak(self) -> float:
		"""SCPI: [SOURce<HW>]:BB:POWer:PEAK \n
		Snippet: value: float = driver.source.bb.power.get_peak() \n
		This command queries the peak power of the baseband signal relative to full scale of 0.5 V (in terms of dB full scale) . \n
			:return: peak: float Range: -145 to 30, Unit: dBfs
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:POWer:PEAK?')
		return Conversions.str_to_float(response)

	def get_rms(self) -> float:
		"""SCPI: [SOURce<HW>]:BB:POWer:RMS \n
		Snippet: value: float = driver.source.bb.power.get_rms() \n
		This command queries the power of the baseband signal relative to full scale of 0.5V (in terms of dB full scale) . \n
			:return: rms: float Range: -145 to 30, Unit: dBfs
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:POWer:RMS?')
		return Conversions.str_to_float(response)



from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup
from ...Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class AllCls:
	"""All commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("all", core, parent)

	def get_measure(self) -> bool:
		"""SCPI: CALibration:ALL:[MEASure] \n
		Snippet: value: bool = driver.calibration.all.get_measure() \n
		Starts all internal adjustments for which no external measuring equipment is required. \n
			:return: all_py: 1| ON| 0| OFF
		"""
		response = self._core.io.query_str('CALibration:ALL:MEASure?')
		return Conversions.str_to_bool(response)



from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup
from ......Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class ObasebandCls:
	"""Obaseband commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("obaseband", core, parent)

	def get_inhibit(self) -> int:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:TRIGger:OBASeband:INHibit \n
		Snippet: value: int = driver.source.bb.arbitrary.trigger.obaseband.get_inhibit() \n
		No command help available \n
			:return: inhibit: No help available
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:TRIGger:OBASeband:INHibit?')
		return Conversions.str_to_int(response)

	def set_inhibit(self, inhibit: int) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:TRIGger:OBASeband:INHibit \n
		Snippet: driver.source.bb.arbitrary.trigger.obaseband.set_inhibit(inhibit = 1) \n
		No command help available \n
			:param inhibit: No help available
		"""
		param = Conversions.decimal_value_to_str(inhibit)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:TRIGger:OBASeband:INHibit {param}')



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions
from .... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class AwgnCls:
	"""Awgn commands group definition. 20 total commands, 4 Subgroups, 5 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("awgn", core, parent)

	@property
	def bandwidth(self):
		"""bandwidth commands group. 1 Sub-classes, 3 commands."""
		if not hasattr(self, '_bandwidth'):
			from .Bandwidth import BandwidthCls
			self._bandwidth = BandwidthCls(self._core, self._cmd_group)
		return self._bandwidth

	@property
	def disp(self):
		"""disp commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_disp'):
			from .Disp import DispCls
			self._disp = DispCls(self._core, self._cmd_group)
		return self._disp

	@property
	def frequency(self):
		"""frequency commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_frequency'):
			from .Frequency import FrequencyCls
			self._frequency = FrequencyCls(self._core, self._cmd_group)
		return self._frequency

	@property
	def power(self):
		"""power commands group. 2 Sub-classes, 3 commands."""
		if not hasattr(self, '_power'):
			from .Power import PowerCls
			self._power = PowerCls(self._core, self._cmd_group)
		return self._power

	def get_brate(self) -> float:
		"""SCPI: [SOURce<HW>]:AWGN:BRATe \n
		Snippet: value: float = driver.source.awgn.get_brate() \n
		Sets the bit rate which is used for calculation of bit energy to noise power ratio from carrier/noise ratio for Digital
		Standard signals. Valid units are bps, kbps and mabps as well as b/s, kb/s and mab/s. \n
			:return: brate: float Range: 400 bps to 250E6 bps
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:AWGN:BRATe?')
		return Conversions.str_to_float(response)

	def set_brate(self, brate: float) -> None:
		"""SCPI: [SOURce<HW>]:AWGN:BRATe \n
		Snippet: driver.source.awgn.set_brate(brate = 1.0) \n
		Sets the bit rate which is used for calculation of bit energy to noise power ratio from carrier/noise ratio for Digital
		Standard signals. Valid units are bps, kbps and mabps as well as b/s, kb/s and mab/s. \n
			:param brate: float Range: 400 bps to 250E6 bps
		"""
		param = Conversions.decimal_value_to_str(brate)
		self._core.io.write(f'SOURce<HwInstance>:AWGN:BRATe {param}')

	def get_cn_ratio(self) -> float:
		"""SCPI: [SOURce<HW>]:AWGN:CNRatio \n
		Snippet: value: float = driver.source.awgn.get_cn_ratio() \n
		In Additive Noise and CW Interferer (SOUR:AWGN:MODE ADD|CW) mode, sets the carrier/interferer ratio. The value range
		depends on the selected AWGN mode. \n
			:return: cn_ratio: float Range: -50 to 40, Unit: dB
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:AWGN:CNRatio?')
		return Conversions.str_to_float(response)

	def set_cn_ratio(self, cn_ratio: float) -> None:
		"""SCPI: [SOURce<HW>]:AWGN:CNRatio \n
		Snippet: driver.source.awgn.set_cn_ratio(cn_ratio = 1.0) \n
		In Additive Noise and CW Interferer (SOUR:AWGN:MODE ADD|CW) mode, sets the carrier/interferer ratio. The value range
		depends on the selected AWGN mode. \n
			:param cn_ratio: float Range: -50 to 40, Unit: dB
		"""
		param = Conversions.decimal_value_to_str(cn_ratio)
		self._core.io.write(f'SOURce<HwInstance>:AWGN:CNRatio {param}')

	def get_en_ratio(self) -> float:
		"""SCPI: [SOURce<HW>]:AWGN:ENRatio \n
		Snippet: value: float = driver.source.awgn.get_en_ratio() \n
		In Additive Noise mode, sets the ratio of bit energy to noise power density. For Digital Standard signals, the bit rate
		used for calculation of Eb/N0 value from C/N value is defined with command SOUR:AWGN:BRAT. \n
			:return: en_ratio: float Range: -50 to depends on hardware, Unit: dB
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:AWGN:ENRatio?')
		return Conversions.str_to_float(response)

	def set_en_ratio(self, en_ratio: float) -> None:
		"""SCPI: [SOURce<HW>]:AWGN:ENRatio \n
		Snippet: driver.source.awgn.set_en_ratio(en_ratio = 1.0) \n
		In Additive Noise mode, sets the ratio of bit energy to noise power density. For Digital Standard signals, the bit rate
		used for calculation of Eb/N0 value from C/N value is defined with command SOUR:AWGN:BRAT. \n
			:param en_ratio: float Range: -50 to depends on hardware, Unit: dB
		"""
		param = Conversions.decimal_value_to_str(en_ratio)
		self._core.io.write(f'SOURce<HwInstance>:AWGN:ENRatio {param}')

	# noinspection PyTypeChecker
	def get_mode(self) -> enums.NoisAwgnMode:
		"""SCPI: [SOURce<HW>]:AWGN:MODE \n
		Snippet: value: enums.NoisAwgnMode = driver.source.awgn.get_mode() \n
		Selects the mode for generating the interfering signal. \n
			:return: mode: ONLY| ADD| CW ADD The AWGN noise signal is added to the baseband signal. ONLY The pure AWGN noise signal is modulated to the carrier. The connection to the baseband is interrupted. CW The sine interfering signal is added to the baseband signal.
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:AWGN:MODE?')
		return Conversions.str_to_scalar_enum(response, enums.NoisAwgnMode)

	def set_mode(self, mode: enums.NoisAwgnMode) -> None:
		"""SCPI: [SOURce<HW>]:AWGN:MODE \n
		Snippet: driver.source.awgn.set_mode(mode = enums.NoisAwgnMode.ADD) \n
		Selects the mode for generating the interfering signal. \n
			:param mode: ONLY| ADD| CW ADD The AWGN noise signal is added to the baseband signal. ONLY The pure AWGN noise signal is modulated to the carrier. The connection to the baseband is interrupted. CW The sine interfering signal is added to the baseband signal.
		"""
		param = Conversions.enum_scalar_to_str(mode, enums.NoisAwgnMode)
		self._core.io.write(f'SOURce<HwInstance>:AWGN:MODE {param}')

	def get_state(self) -> bool:
		"""SCPI: [SOURce<HW>]:AWGN:STATe \n
		Snippet: value: bool = driver.source.awgn.get_state() \n
		Activates or deactivates the white noise (AWGN = Averaged White Gaussian Noise) . Use the command [:SOURce<hw>]:AWGN:MODE
		to define the mode the AWGN generator is working in. \n
			:return: state: 0| 1| OFF| ON
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:AWGN:STATe?')
		return Conversions.str_to_bool(response)

	def set_state(self, state: bool) -> None:
		"""SCPI: [SOURce<HW>]:AWGN:STATe \n
		Snippet: driver.source.awgn.set_state(state = False) \n
		Activates or deactivates the white noise (AWGN = Averaged White Gaussian Noise) . Use the command [:SOURce<hw>]:AWGN:MODE
		to define the mode the AWGN generator is working in. \n
			:param state: 0| 1| OFF| ON
		"""
		param = Conversions.bool_to_str(state)
		self._core.io.write(f'SOURce<HwInstance>:AWGN:STATe {param}')

	def clone(self) -> 'AwgnCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = AwgnCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from .......Internal.Core import Core
from .......Internal.CommandsGroup import CommandsGroup


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class ExecuteCls:
	"""Execute commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("execute", core, parent)

	def set(self) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WSEGment:NEXT:EXECute \n
		Snippet: driver.source.bb.arbitrary.wsegment.next.execute.set() \n
		Triggers manually switchover to the subsequent segment in the mutli segment file. A manual trigger can be executed only
		when an internal next segment source (BB:ARB:WSEG:NEXT:SOUR INT) has been selected. To perform a switchover to any
		segment within the multi segment file, select the next segment with the command BB:ARB:WSEG:NEXT. This command is
		disabled, if a sequencing play list is enabled. \n
		"""
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:WSEGment:NEXT:EXECute')

	def set_with_opc(self, opc_timeout_ms: int = -1) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WSEGment:NEXT:EXECute \n
		Snippet: driver.source.bb.arbitrary.wsegment.next.execute.set_with_opc() \n
		Triggers manually switchover to the subsequent segment in the mutli segment file. A manual trigger can be executed only
		when an internal next segment source (BB:ARB:WSEG:NEXT:SOUR INT) has been selected. To perform a switchover to any
		segment within the multi segment file, select the next segment with the command BB:ARB:WSEG:NEXT. This command is
		disabled, if a sequencing play list is enabled. \n
		Same as set, but waits for the operation to complete before continuing further. Use the RsSgt.utilities.opc_timeout_set() to set the timeout value. \n
			:param opc_timeout_ms: Maximum time to wait in milliseconds, valid only for this call."""
		self._core.io.write_with_opc(f'SOURce<HwInstance>:BB:ARBitrary:WSEGment:NEXT:EXECute', opc_timeout_ms)



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions
from ..... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class PowerCls:
	"""Power commands group definition. 7 total commands, 2 Subgroups, 3 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("power", core, parent)

	@property
	def noise(self):
		"""noise commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_noise'):
			from .Noise import NoiseCls
			self._noise = NoiseCls(self._core, self._cmd_group)
		return self._noise

	@property
	def sum(self):
		"""sum commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_sum'):
			from .Sum import SumCls
			self._sum = SumCls(self._core, self._cmd_group)
		return self._sum

	def get_carrier(self) -> float:
		"""SCPI: [SOURce<HW>]:AWGN:POWer:CARRier \n
		Snippet: value: float = driver.source.awgn.power.get_carrier() \n
		available for Additive Noise and CW Interferer (SOUR:AWGN:MODE ADD|CW) modes
			INTRO_CMD_HELP: Sets/queries the carrier or signal power depending on the selected reference mode. \n
			- SOUR:AWGN:POW:RMOD CARR Sets the carrier power. The power of the noise signal is derived from the entered C/N value.
			- SOUR:AWGN:POW:RMOD NOIS queries the carrier power which is derived from the entered C/N value. The noise power is set with command SOUR:AWGN:POW:NOISe. \n
			:return: carrier: float Unit: dBm
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:AWGN:POWer:CARRier?')
		return Conversions.str_to_float(response)

	def set_carrier(self, carrier: float) -> None:
		"""SCPI: [SOURce<HW>]:AWGN:POWer:CARRier \n
		Snippet: driver.source.awgn.power.set_carrier(carrier = 1.0) \n
		available for Additive Noise and CW Interferer (SOUR:AWGN:MODE ADD|CW) modes
			INTRO_CMD_HELP: Sets/queries the carrier or signal power depending on the selected reference mode. \n
			- SOUR:AWGN:POW:RMOD CARR Sets the carrier power. The power of the noise signal is derived from the entered C/N value.
			- SOUR:AWGN:POW:RMOD NOIS queries the carrier power which is derived from the entered C/N value. The noise power is set with command SOUR:AWGN:POW:NOISe. \n
			:param carrier: float Unit: dBm
		"""
		param = Conversions.decimal_value_to_str(carrier)
		self._core.io.write(f'SOURce<HwInstance>:AWGN:POWer:CARRier {param}')

	# noinspection PyTypeChecker
	def get_mode(self) -> enums.NoisAwgnPowMode:
		"""SCPI: [SOURce<HW>]:AWGN:POWer:MODE \n
		Snippet: value: enums.NoisAwgnPowMode = driver.source.awgn.power.get_mode() \n
		In Additive Noise (SOUR:AWGN:MODE ADD) mode, selects the mode for setting the noise power. \n
			:return: mode: CN| SN | EN CN|SN The noise power is set on the basis of the value entered for the carrier/noise or signal/noise ratio (SOURce:AWGN:CNRatio|SNRatio) . EN The noise power is set on the basis of the value entered for the ratio of bit energy to noise power density (AWGN:ENR) .
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:AWGN:POWer:MODE?')
		return Conversions.str_to_scalar_enum(response, enums.NoisAwgnPowMode)

	def set_mode(self, mode: enums.NoisAwgnPowMode) -> None:
		"""SCPI: [SOURce<HW>]:AWGN:POWer:MODE \n
		Snippet: driver.source.awgn.power.set_mode(mode = enums.NoisAwgnPowMode.CN) \n
		In Additive Noise (SOUR:AWGN:MODE ADD) mode, selects the mode for setting the noise power. \n
			:param mode: CN| SN | EN CN|SN The noise power is set on the basis of the value entered for the carrier/noise or signal/noise ratio (SOURce:AWGN:CNRatio|SNRatio) . EN The noise power is set on the basis of the value entered for the ratio of bit energy to noise power density (AWGN:ENR) .
		"""
		param = Conversions.enum_scalar_to_str(mode, enums.NoisAwgnPowMode)
		self._core.io.write(f'SOURce<HwInstance>:AWGN:POWer:MODE {param}')

	# noinspection PyTypeChecker
	def get_rmode(self) -> enums.NoisAwgnPowRefMode:
		"""SCPI: [SOURce<HW>]:AWGN:POWer:RMODe \n
		Snippet: value: enums.NoisAwgnPowRefMode = driver.source.awgn.power.get_rmode() \n
		In Additive Noise and CW Interferer (SOUR:AWGN:MODE ADD|CW) modes and Display Mode set to RF (AWGN:DISP:MODE RF) ,
		selects the mode for setting the interfering signal. \n
			:return: rmode: CARRier| NOISe CARRier The carrier power is kept constant when the C/N value or Eb/N0 value is changed. The noise power is adjusted. NOISe The noise power is kept constant when the C/N value or Eb/N0 value is changed. The carrier power is adjusted.
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:AWGN:POWer:RMODe?')
		return Conversions.str_to_scalar_enum(response, enums.NoisAwgnPowRefMode)

	def set_rmode(self, rmode: enums.NoisAwgnPowRefMode) -> None:
		"""SCPI: [SOURce<HW>]:AWGN:POWer:RMODe \n
		Snippet: driver.source.awgn.power.set_rmode(rmode = enums.NoisAwgnPowRefMode.CARRier) \n
		In Additive Noise and CW Interferer (SOUR:AWGN:MODE ADD|CW) modes and Display Mode set to RF (AWGN:DISP:MODE RF) ,
		selects the mode for setting the interfering signal. \n
			:param rmode: CARRier| NOISe CARRier The carrier power is kept constant when the C/N value or Eb/N0 value is changed. The noise power is adjusted. NOISe The noise power is kept constant when the C/N value or Eb/N0 value is changed. The carrier power is adjusted.
		"""
		param = Conversions.enum_scalar_to_str(rmode, enums.NoisAwgnPowRefMode)
		self._core.io.write(f'SOURce<HwInstance>:AWGN:POWer:RMODe {param}')

	def clone(self) -> 'PowerCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = PowerCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions
from ....Internal.Utilities import trim_str_response


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class IqModulatorCls:
	"""IqModulator commands group definition. 5 total commands, 2 Subgroups, 3 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("iqModulator", core, parent)

	@property
	def bband(self):
		"""bband commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_bband'):
			from .Bband import BbandCls
			self._bband = BbandCls(self._core, self._cmd_group)
		return self._bband

	@property
	def iqModulator(self):
		"""iqModulator commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_iqModulator'):
			from .IqModulator import IqModulatorCls
			self._iqModulator = IqModulatorCls(self._core, self._cmd_group)
		return self._iqModulator

	def get_full(self) -> bool:
		"""SCPI: CALibration:IQModulator:FULL \n
		Snippet: value: bool = driver.calibration.iqModulator.get_full() \n
		Starts the adjustment of the I/Q modulator for the entire frequency range. The I/Q modulator is adjusted with respect to
		carrier leakage, I/Q imbalance and quadrature. \n
			:return: modulator: 1| ON| 0| OFF
		"""
		response = self._core.io.query_str('CALibration:IQModulator:FULL?')
		return Conversions.str_to_bool(response)

	def get_local(self) -> bool:
		"""SCPI: CALibration:IQModulator:LOCal \n
		Snippet: value: bool = driver.calibration.iqModulator.get_local() \n
		Starts the adjustment of the I/Q modulator for the current frequency. The I/Q modulator is adjusted with respect to
		carrier leakage, I/Q imbalance and quadrature. This adjustment is only possible when OUTPut[:STATe] ON and
		[:SOURce]:IQ:STATe ON. \n
			:return: cal_modulator_loc_error: No help available
		"""
		response = self._core.io.query_str('CALibration:IQModulator:LOCal?')
		return Conversions.str_to_bool(response)

	def get_temperature(self) -> str:
		"""SCPI: CALibration:IQModulator:TEMPerature \n
		Snippet: value: str = driver.calibration.iqModulator.get_temperature() \n
		Queries the delta temperature since the last performed adjustment. \n
			:return: temperature: string
		"""
		response = self._core.io.query_str('CALibration:IQModulator:TEMPerature?')
		return trim_str_response(response)

	def clone(self) -> 'IqModulatorCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = IqModulatorCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class FrequencyCls:
	"""Frequency commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("frequency", core, parent)

	def get_result(self) -> float:
		"""SCPI: [SOURce<HW>]:AWGN:FREQuency:RESult \n
		Snippet: value: float = driver.source.awgn.frequency.get_result() \n
		In CW inerferer mode, queries the actual frequency of the sine. The actual frequency may differ from the desired
		frequency, since the resolution is limited to 0.7 Hz. \n
			:return: result: float Range: -40E6 to 40E6
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:AWGN:FREQuency:RESult?')
		return Conversions.str_to_float(response)

	def get_target(self) -> float:
		"""SCPI: [SOURce<HW>]:AWGN:FREQuency:TARGet \n
		Snippet: value: float = driver.source.awgn.frequency.get_target() \n
		Sets the desired frequency of the sine in AWGN:MODE CW mode. The resulting frequency may differ from the desired
		frequency because of the limited frequency resolution of 0.7 Hz. \n
			:return: target: float Range: -40E6 to 40E6
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:AWGN:FREQuency:TARGet?')
		return Conversions.str_to_float(response)

	def set_target(self, target: float) -> None:
		"""SCPI: [SOURce<HW>]:AWGN:FREQuency:TARGet \n
		Snippet: driver.source.awgn.frequency.set_target(target = 1.0) \n
		Sets the desired frequency of the sine in AWGN:MODE CW mode. The resulting frequency may differ from the desired
		frequency because of the limited frequency resolution of 0.7 Hz. \n
			:param target: float Range: -40E6 to 40E6
		"""
		param = Conversions.decimal_value_to_str(target)
		self._core.io.write(f'SOURce<HwInstance>:AWGN:FREQuency:TARGet {param}')



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions
from ..... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class OptimizationCls:
	"""Optimization commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("optimization", core, parent)

	# noinspection PyTypeChecker
	def get_mode(self) -> enums.BbImpOptMode:
		"""SCPI: [SOURce<HW>]:BB:IMPairment:OPTimization:MODE \n
		Snippet: value: enums.BbImpOptMode = driver.source.bb.impairment.optimization.get_mode() \n
		This command sets the optimization mode. \n
			:return: mode: FAST| QHIGh FAST Optimization is reached by compensation for I/Q skew. QHIGh Optimization is reached by compensation for I/Q skew and frequency response correction.
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:IMPairment:OPTimization:MODE?')
		return Conversions.str_to_scalar_enum(response, enums.BbImpOptMode)

	def set_mode(self, mode: enums.BbImpOptMode) -> None:
		"""SCPI: [SOURce<HW>]:BB:IMPairment:OPTimization:MODE \n
		Snippet: driver.source.bb.impairment.optimization.set_mode(mode = enums.BbImpOptMode.FAST) \n
		This command sets the optimization mode. \n
			:param mode: FAST| QHIGh FAST Optimization is reached by compensation for I/Q skew. QHIGh Optimization is reached by compensation for I/Q skew and frequency response correction.
		"""
		param = Conversions.enum_scalar_to_str(mode, enums.BbImpOptMode)
		self._core.io.write(f'SOURce<HwInstance>:BB:IMPairment:OPTimization:MODE {param}')



from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup
from ......Internal import Conversions
from ......Internal.Utilities import trim_str_response
from ......Internal.RepeatedCapability import RepeatedCapability
from ...... import repcap


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class McarrierCls:
	"""Mcarrier commands group definition. 38 total commands, 9 Subgroups, 4 group commands
	Repeated Capability: Carrier, default value after init: Carrier.Nr1"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("mcarrier", core, parent)
		self._cmd_group.rep_cap = RepeatedCapability(self._cmd_group.group_name, 'repcap_carrier_get', 'repcap_carrier_set', repcap.Carrier.Nr1)

	def repcap_carrier_set(self, carrier: repcap.Carrier) -> None:
		"""Repeated Capability default value numeric suffix.
		This value is used, if you do not explicitely set it in the child set/get methods, or if you leave it to Carrier.Default
		Default value after init: Carrier.Nr1"""
		self._cmd_group.set_repcap_enum_value(carrier)

	def repcap_carrier_get(self) -> repcap.Carrier:
		"""Returns the current default repeated capability for the child set/get methods"""
		# noinspection PyTypeChecker
		return self._cmd_group.get_repcap_enum_value()

	@property
	def carrier(self):
		"""carrier commands group. 7 Sub-classes, 3 commands."""
		if not hasattr(self, '_carrier'):
			from .Carrier import CarrierCls
			self._carrier = CarrierCls(self._core, self._cmd_group)
		return self._carrier

	@property
	def cfactor(self):
		"""cfactor commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_cfactor'):
			from .Cfactor import CfactorCls
			self._cfactor = CfactorCls(self._core, self._cmd_group)
		return self._cfactor

	@property
	def clipping(self):
		"""clipping commands group. 0 Sub-classes, 3 commands."""
		if not hasattr(self, '_clipping'):
			from .Clipping import ClippingCls
			self._clipping = ClippingCls(self._core, self._cmd_group)
		return self._clipping

	@property
	def cload(self):
		"""cload commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_cload'):
			from .Cload import CloadCls
			self._cload = CloadCls(self._core, self._cmd_group)
		return self._cload

	@property
	def create(self):
		"""create commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_create'):
			from .Create import CreateCls
			self._create = CreateCls(self._core, self._cmd_group)
		return self._create

	@property
	def edit(self):
		"""edit commands group. 1 Sub-classes, 0 commands."""
		if not hasattr(self, '_edit'):
			from .Edit import EditCls
			self._edit = EditCls(self._core, self._cmd_group)
		return self._edit

	@property
	def power(self):
		"""power commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_power'):
			from .Power import PowerCls
			self._power = PowerCls(self._core, self._cmd_group)
		return self._power

	@property
	def setting(self):
		"""setting commands group. 1 Sub-classes, 2 commands."""
		if not hasattr(self, '_setting'):
			from .Setting import SettingCls
			self._setting = SettingCls(self._core, self._cmd_group)
		return self._setting

	@property
	def time(self):
		"""time commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_time'):
			from .Time import TimeCls
			self._time = TimeCls(self._core, self._cmd_group)
		return self._time

	def get_clock(self) -> float:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:CLOCk \n
		Snippet: value: float = driver.source.bb.arbitrary.mcarrier.get_clock() \n
		The command queries the resulting sample rate at which the multi carrier waveform is output by the arbitrary waveform
		generator. The output clock rate depends on the number of carriers, carrier spacing and input sample rate of the leftmost
		or rightmost carriers. \n
			:return: clock: float Range: 400 to Max
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:MCARrier:CLOCk?')
		return Conversions.str_to_float(response)

	def get_ofile(self) -> str:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:OFILe \n
		Snippet: value: str = driver.source.bb.arbitrary.mcarrier.get_ofile() \n
		This command defines the output file name for the multi carrier waveform. This file name is used when a waveform is
		calculated (command SOUR:BB:ARB:MCAR:CLOad or SOUR:BB:ARB:MCAR:CREate) . The file extension is *.wv. \n
			:return: ofile: string
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:MCARrier:OFILe?')
		return trim_str_response(response)

	def set_ofile(self, ofile: str) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:OFILe \n
		Snippet: driver.source.bb.arbitrary.mcarrier.set_ofile(ofile = '1') \n
		This command defines the output file name for the multi carrier waveform. This file name is used when a waveform is
		calculated (command SOUR:BB:ARB:MCAR:CLOad or SOUR:BB:ARB:MCAR:CREate) . The file extension is *.wv. \n
			:param ofile: string
		"""
		param = Conversions.value_to_quoted_str(ofile)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:MCARrier:OFILe {param}')

	def preset(self) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:PRESet \n
		Snippet: driver.source.bb.arbitrary.mcarrier.preset() \n
		Sets all ARB multi carrier parameters to their default values. \n
		"""
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:MCARrier:PRESet')

	def preset_with_opc(self, opc_timeout_ms: int = -1) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:PRESet \n
		Snippet: driver.source.bb.arbitrary.mcarrier.preset_with_opc() \n
		Sets all ARB multi carrier parameters to their default values. \n
		Same as preset, but waits for the operation to complete before continuing further. Use the RsSgt.utilities.opc_timeout_set() to set the timeout value. \n
			:param opc_timeout_ms: Maximum time to wait in milliseconds, valid only for this call."""
		self._core.io.write_with_opc(f'SOURce<HwInstance>:BB:ARBitrary:MCARrier:PRESet', opc_timeout_ms)

	def get_samples(self) -> int:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:SAMPles \n
		Snippet: value: int = driver.source.bb.arbitrary.mcarrier.get_samples() \n
		The command queries the resulting file size. The file size is returned in samples. \n
			:return: samples: integer Range: 0 to INT_MAX
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:MCARrier:SAMPles?')
		return Conversions.str_to_int(response)

	def clone(self) -> 'McarrierCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = McarrierCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class PackageCls:
	"""Package commands group definition. 3 total commands, 3 Subgroups, 0 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("package", core, parent)

	@property
	def chartDisplay(self):
		"""chartDisplay commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_chartDisplay'):
			from .ChartDisplay import ChartDisplayCls
			self._chartDisplay = ChartDisplayCls(self._core, self._cmd_group)
		return self._chartDisplay

	@property
	def guiFramework(self):
		"""guiFramework commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_guiFramework'):
			from .GuiFramework import GuiFrameworkCls
			self._guiFramework = GuiFrameworkCls(self._core, self._cmd_group)
		return self._guiFramework

	@property
	def qt(self):
		"""qt commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_qt'):
			from .Qt import QtCls
			self._qt = QtCls(self._core, self._cmd_group)
		return self._qt

	def clone(self) -> 'PackageCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = PackageCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from ........Internal.Core import Core
from ........Internal.CommandsGroup import CommandsGroup
from ........Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class ValueCls:
	"""Value commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("value", core, parent)

	def get_level(self) -> float:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:VCC:VALue:LEVel \n
		Snippet: value: float = driver.source.iq.output.analog.envelope.vcc.value.get_level() \n
		Queries the Vcc value of the current RMS power level (operating point) . \n
			:return: vcc_for_rf_level: float Range: 0 to 38
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:VCC:VALue:LEVel?')
		return Conversions.str_to_float(response)

	def get_pep(self) -> float:
		"""SCPI: [SOURce<HW>]:IQ:OUTPut:[ANALog]:ENVelope:VCC:VALue:PEP \n
		Snippet: value: float = driver.source.iq.output.analog.envelope.vcc.value.get_pep() \n
		Queries the Vcc value of the current PEP of the generated RF signal. \n
			:return: vcc_for_crt_pep: float Range: 0 to 38
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:OUTPut:ANALog:ENVelope:VCC:VALue:PEP?')
		return Conversions.str_to_float(response)



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class EepromCls:
	"""Eeprom commands group definition. 2 total commands, 1 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("eeprom", core, parent)

	@property
	def data(self):
		"""data commands group. 1 Sub-classes, 0 commands."""
		if not hasattr(self, '_data'):
			from .Data import DataCls
			self._data = DataCls(self._core, self._cmd_group)
		return self._data

	def delete(self) -> None:
		"""SCPI: DIAGnostic<HW>:EEPRom:DELete \n
		Snippet: driver.diagnostic.eeprom.delete() \n
		No command help available \n
		"""
		self._core.io.write(f'DIAGnostic<HwInstance>:EEPRom:DELete')

	def delete_with_opc(self, opc_timeout_ms: int = -1) -> None:
		"""SCPI: DIAGnostic<HW>:EEPRom:DELete \n
		Snippet: driver.diagnostic.eeprom.delete_with_opc() \n
		No command help available \n
		Same as delete, but waits for the operation to complete before continuing further. Use the RsSgt.utilities.opc_timeout_set() to set the timeout value. \n
			:param opc_timeout_ms: Maximum time to wait in milliseconds, valid only for this call."""
		self._core.io.write_with_opc(f'DIAGnostic<HwInstance>:EEPRom:DELete', opc_timeout_ms)

	def clone(self) -> 'EepromCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = EepromCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class ArbitraryCls:
	"""Arbitrary commands group definition. 130 total commands, 9 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("arbitrary", core, parent)

	@property
	def cfr(self):
		"""cfr commands group. 2 Sub-classes, 12 commands."""
		if not hasattr(self, '_cfr'):
			from .Cfr import CfrCls
			self._cfr = CfrCls(self._core, self._cmd_group)
		return self._cfr

	@property
	def clock(self):
		"""clock commands group. 1 Sub-classes, 4 commands."""
		if not hasattr(self, '_clock'):
			from .Clock import ClockCls
			self._clock = ClockCls(self._core, self._cmd_group)
		return self._clock

	@property
	def mcarrier(self):
		"""mcarrier commands group. 9 Sub-classes, 4 commands."""
		if not hasattr(self, '_mcarrier'):
			from .Mcarrier import McarrierCls
			self._mcarrier = McarrierCls(self._core, self._cmd_group)
		return self._mcarrier

	@property
	def pramp(self):
		"""pramp commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_pramp'):
			from .Pramp import PrampCls
			self._pramp = PrampCls(self._core, self._cmd_group)
		return self._pramp

	@property
	def signal(self):
		"""signal commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_signal'):
			from .Signal import SignalCls
			self._signal = SignalCls(self._core, self._cmd_group)
		return self._signal

	@property
	def trigger(self):
		"""trigger commands group. 6 Sub-classes, 7 commands."""
		if not hasattr(self, '_trigger'):
			from .Trigger import TriggerCls
			self._trigger = TriggerCls(self._core, self._cmd_group)
		return self._trigger

	@property
	def tsignal(self):
		"""tsignal commands group. 4 Sub-classes, 0 commands."""
		if not hasattr(self, '_tsignal'):
			from .Tsignal import TsignalCls
			self._tsignal = TsignalCls(self._core, self._cmd_group)
		return self._tsignal

	@property
	def waveform(self):
		"""waveform commands group. 2 Sub-classes, 5 commands."""
		if not hasattr(self, '_waveform'):
			from .Waveform import WaveformCls
			self._waveform = WaveformCls(self._core, self._cmd_group)
		return self._waveform

	@property
	def wsegment(self):
		"""wsegment commands group. 3 Sub-classes, 5 commands."""
		if not hasattr(self, '_wsegment'):
			from .Wsegment import WsegmentCls
			self._wsegment = WsegmentCls(self._core, self._cmd_group)
		return self._wsegment

	def preset(self) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:PRESet \n
		Snippet: driver.source.bb.arbitrary.preset() \n
		Sets all ARB generator parameters to their default values. \n
		"""
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:PRESet')

	def preset_with_opc(self, opc_timeout_ms: int = -1) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:PRESet \n
		Snippet: driver.source.bb.arbitrary.preset_with_opc() \n
		Sets all ARB generator parameters to their default values. \n
		Same as preset, but waits for the operation to complete before continuing further. Use the RsSgt.utilities.opc_timeout_set() to set the timeout value. \n
			:param opc_timeout_ms: Maximum time to wait in milliseconds, valid only for this call."""
		self._core.io.write_with_opc(f'SOURce<HwInstance>:BB:ARBitrary:PRESet', opc_timeout_ms)

	def get_state(self) -> bool:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:STATe \n
		Snippet: value: bool = driver.source.bb.arbitrary.get_state() \n
		Activates the standard and deactivates all the other digital standards and digital modulation modes in the same path. You
		have to selecta an waveform first. \n
			:return: state: 0| 1| OFF| ON
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:STATe?')
		return Conversions.str_to_bool(response)

	def set_state(self, state: bool) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:STATe \n
		Snippet: driver.source.bb.arbitrary.set_state(state = False) \n
		Activates the standard and deactivates all the other digital standards and digital modulation modes in the same path. You
		have to selecta an waveform first. \n
			:param state: 0| 1| OFF| ON
		"""
		param = Conversions.bool_to_str(state)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:STATe {param}')

	def clone(self) -> 'ArbitraryCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = ArbitraryCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group
